# -*- coding: utf-8 -*-
# This file is part of AYAB.
#
#    AYAB is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    AYAB is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with AYAB.  If not, see <http://www.gnu.org/licenses/>.
#
#    Copyright 2014 Sebastian Oliva, Christian Obersteiner, Andreas Müller
#    https://bitbucket.org/chris007de/ayab-apparat/

import pytest
import serial
import unittest
from ayab.plugins.ayab_plugin.ayab_communication import AyabCommunication
from mock import patch


class TestCommunication(unittest.TestCase):

  def setUp(self):
    self.dummy_serial = serial.serial_for_url("loop://logging=debug")
    self.comm_dummy = AyabCommunication(self.dummy_serial)

  def test_close_serial(self):
      before = self.dummy_serial.isOpen()
      assert before
      self.comm_dummy.close_serial()
      after = self.dummy_serial.isOpen()
      assert after == False

  def test_open_serial(self):
    with patch.object(serial,'Serial') as mock_method:
      mock_method.return_value = object()
      self.ayabCom = AyabCommunication()
      openStatus = self.ayabCom.open_serial('dummyPortname')
      assert openStatus
      mock_method.assert_called_once_with('dummyPortname',115200)

    with patch.object(serial,'Serial') as mock_method:
      with pytest.raises(Exception) as excinfo:
        mock_method.side_effect = serial.SerialException()
        self.ayabCom = AyabCommunication()
        openStatus = self.ayabCom.open_serial('dummyPortname')
      assert "CommunicationException" in str(excinfo.type)
      mock_method.assert_called_once_with('dummyPortname',115200)

  def test_req_start(self):
    start_val, end_val = 0, 10
    self.comm_dummy.req_start(start_val, end_val)
    byte_array = bytearray([0x01, start_val, end_val, 0x0a, 0x0d])
    bytes_read = self.dummy_serial.read(len(byte_array))
    self.assertEqual(bytes_read, byte_array)

  def test_req_info(self):
    self.comm_dummy.req_info()
    byte_array = bytearray([0x03, 0x0a, 0x0d])
    bytes_read = self.dummy_serial.read(len(byte_array))
    assert bytes_read == byte_array

  def test_cnf_line(self):
    lineNumber = 13
    lineData   = chr(0xAB)
    flags      = 0x12
    crc8       = 0x57
    self.comm_dummy.cnf_line(lineNumber, lineData, flags, crc8)
    byte_array = bytearray([0x42, lineNumber, lineData, flags, crc8, 0x0a, 0x0d])
    bytes_read = self.dummy_serial.read(len(byte_array))
    assert bytes_read == byte_array

  def test_read_line(self):
    byte_start_val = 0x01
    byte_end_val = 0x12
    byte_array = bytearray([0xC1, byte_start_val, byte_end_val, 0x0a, 0x0d])
    bytes_wrote = self.dummy_serial.write(byte_array)
    bytes_read = self.comm_dummy.read_line()
    assert bytes_read == byte_array




# -*- coding: utf-8 -*-
# This file is part of AYAB.
#
#    AYAB is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    AYAB is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with AYAB.  If not, see <http://www.gnu.org/licenses/>.
#
#    Copyright 2013, 2014 Sebastian Oliva, Christian Obersteiner, Andreas Müller
#    https://bitbucket.org/chris007de/ayab-apparat/

import time
from PyQt4 import QtGui, QtCore
from ayab.plugins.knitting_plugin import KnittingPlugin
import logging

try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    def _fromUtf8(s):
        return s

try:
    _encoding = QtGui.QApplication.UnicodeUTF8

    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig, _encoding)
except AttributeError:

    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig)


class DummyKnittingPlugin(KnittingPlugin):

  def onknit(self, e):
    logging.debug("called onknit on DummyKnittingPlugin")
    # Simulating a blocking task.
    for i in range(self._cycle_ammount):
      percent = (i / float(self._cycle_ammount))*100
      print percent
      self.parent_ui.emit(QtCore.SIGNAL('updateProgress(int)'), int(percent))
      time.sleep(0.1)
    self.finish()
    return True

  def onconfigure(self, e):
    logging.debug("Called onconfigure on DummyKnittingPlugin")
    self._cycle_ammount = 20
    return

  def onfinish(self, e):
    logging.info("Finished Knitting")
    pass

  def setup_ui(self, parent_ui):
    self.parent_ui = parent_ui
    dock = parent_ui.ui.knitting_options_dock
    self.qwidget = QtGui.QWidget(dock)
    self.configure_button = QtGui.QPushButton(self.qwidget)
    self.configure_button.setObjectName(_fromUtf8("configure_button"))
    self.configure_button.setText(_translate("DockWidget", "Configure", None))
    self.configure_button.clicked.connect(self.__conf_button_function)
    dock.setWidget(self.qwidget)

  def cleanup_ui(self, parent_ui):
    self.parent_ui = parent_ui
    dock = parent_ui.ui.knitting_options_dock
    cleaner = QtCore.QObjectCleanupHandler()
    cleaner.add(self.qwidget)
    self.__qw = QtGui.QWidget()
    dock.setWidget(self.__qw)

  def __conf_button_function(self):
    self.configure()

  def get_configuration_from_ui(self, ui):
    pass

  def __init__(self):
    super(DummyKnittingPlugin, self).__init__({})



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'ayab/plugins/ayab_plugin/ayab_options.ui'
#
# Created: Wed Feb 11 17:33:45 2015
#      by: PyQt4 UI code generator 4.10.4
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    def _fromUtf8(s):
        return s

try:
    _encoding = QtGui.QApplication.UnicodeUTF8
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig, _encoding)
except AttributeError:
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig)

class Ui_DockWidget(object):
    def setupUi(self, DockWidget):
        DockWidget.setObjectName(_fromUtf8("DockWidget"))
        DockWidget.resize(237, 530)
        DockWidget.setFeatures(QtGui.QDockWidget.NoDockWidgetFeatures)
        self.ayab_config = QtGui.QWidget()
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Preferred, QtGui.QSizePolicy.Maximum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.ayab_config.sizePolicy().hasHeightForWidth())
        self.ayab_config.setSizePolicy(sizePolicy)
        self.ayab_config.setObjectName(_fromUtf8("ayab_config"))
        self.verticalLayout = QtGui.QVBoxLayout(self.ayab_config)
        self.verticalLayout.setObjectName(_fromUtf8("verticalLayout"))
        self.groupBox = QtGui.QGroupBox(self.ayab_config)
        self.groupBox.setObjectName(_fromUtf8("groupBox"))
        self.horizontalLayout_3 = QtGui.QHBoxLayout(self.groupBox)
        self.horizontalLayout_3.setObjectName(_fromUtf8("horizontalLayout_3"))
        self.serial_port_dropdown = QtGui.QComboBox(self.groupBox)
        self.serial_port_dropdown.setObjectName(_fromUtf8("serial_port_dropdown"))
        self.horizontalLayout_3.addWidget(self.serial_port_dropdown)
        self.refresh_ports_button = QtGui.QPushButton(self.groupBox)
        self.refresh_ports_button.setObjectName(_fromUtf8("refresh_ports_button"))
        self.horizontalLayout_3.addWidget(self.refresh_ports_button)
        self.verticalLayout.addWidget(self.groupBox)
        self.label_6 = QtGui.QLabel(self.ayab_config)
        self.label_6.setObjectName(_fromUtf8("label_6"))
        self.verticalLayout.addWidget(self.label_6)
        self.color_edit = QtGui.QSpinBox(self.ayab_config)
        self.color_edit.setMinimum(2)
        self.color_edit.setMaximum(6)
        self.color_edit.setObjectName(_fromUtf8("color_edit"))
        self.verticalLayout.addWidget(self.color_edit)
        self.label_5 = QtGui.QLabel(self.ayab_config)
        self.label_5.setObjectName(_fromUtf8("label_5"))
        self.verticalLayout.addWidget(self.label_5)
        self.start_line_edit = QtGui.QSpinBox(self.ayab_config)
        self.start_line_edit.setSuffix(_fromUtf8(""))
        self.start_line_edit.setMinimum(0)
        self.start_line_edit.setMaximum(256)
        self.start_line_edit.setObjectName(_fromUtf8("start_line_edit"))
        self.verticalLayout.addWidget(self.start_line_edit)
        self.infRepeat_checkbox = QtGui.QCheckBox(self.ayab_config)
        self.infRepeat_checkbox.setObjectName(_fromUtf8("infRepeat_checkbox"))
        self.verticalLayout.addWidget(self.infRepeat_checkbox)
        self.line = QtGui.QFrame(self.ayab_config)
        self.line.setFrameShape(QtGui.QFrame.HLine)
        self.line.setFrameShadow(QtGui.QFrame.Sunken)
        self.line.setObjectName(_fromUtf8("line"))
        self.verticalLayout.addWidget(self.line)
        self.gbox_startneedle = QtGui.QGroupBox(self.ayab_config)
        self.gbox_startneedle.setMinimumSize(QtCore.QSize(0, 60))
        self.gbox_startneedle.setObjectName(_fromUtf8("gbox_startneedle"))
        self.start_needle_color = QtGui.QComboBox(self.gbox_startneedle)
        self.start_needle_color.setGeometry(QtCore.QRect(100, 20, 81, 31))
        self.start_needle_color.setObjectName(_fromUtf8("start_needle_color"))
        self.start_needle_color.addItem(_fromUtf8(""))
        self.start_needle_color.addItem(_fromUtf8(""))
        self.start_needle_edit = QtGui.QSpinBox(self.gbox_startneedle)
        self.start_needle_edit.setGeometry(QtCore.QRect(50, 20, 51, 31))
        self.start_needle_edit.setPrefix(_fromUtf8(""))
        self.start_needle_edit.setMinimum(1)
        self.start_needle_edit.setMaximum(100)
        self.start_needle_edit.setProperty("value", 20)
        self.start_needle_edit.setObjectName(_fromUtf8("start_needle_edit"))
        self.verticalLayout.addWidget(self.gbox_startneedle)
        self.gbox_stopneedle = QtGui.QGroupBox(self.ayab_config)
        self.gbox_stopneedle.setMinimumSize(QtCore.QSize(0, 60))
        self.gbox_stopneedle.setObjectName(_fromUtf8("gbox_stopneedle"))
        self.stop_needle_color = QtGui.QComboBox(self.gbox_stopneedle)
        self.stop_needle_color.setGeometry(QtCore.QRect(100, 20, 81, 31))
        self.stop_needle_color.setObjectName(_fromUtf8("stop_needle_color"))
        self.stop_needle_color.addItem(_fromUtf8(""))
        self.stop_needle_color.addItem(_fromUtf8(""))
        self.stop_needle_edit = QtGui.QSpinBox(self.gbox_stopneedle)
        self.stop_needle_edit.setGeometry(QtCore.QRect(50, 20, 51, 31))
        self.stop_needle_edit.setPrefix(_fromUtf8(""))
        self.stop_needle_edit.setMinimum(1)
        self.stop_needle_edit.setMaximum(100)
        self.stop_needle_edit.setProperty("value", 20)
        self.stop_needle_edit.setObjectName(_fromUtf8("stop_needle_edit"))
        self.verticalLayout.addWidget(self.gbox_stopneedle)
        self.label_3 = QtGui.QLabel(self.ayab_config)
        self.label_3.setObjectName(_fromUtf8("label_3"))
        self.verticalLayout.addWidget(self.label_3)
        self.alignment_combo_box = QtGui.QComboBox(self.ayab_config)
        self.alignment_combo_box.setObjectName(_fromUtf8("alignment_combo_box"))
        self.alignment_combo_box.addItem(_fromUtf8(""))
        self.alignment_combo_box.addItem(_fromUtf8(""))
        self.alignment_combo_box.addItem(_fromUtf8(""))
        self.verticalLayout.addWidget(self.alignment_combo_box)
        self.label_4 = QtGui.QLabel(self.ayab_config)
        self.label_4.setObjectName(_fromUtf8("label_4"))
        self.verticalLayout.addWidget(self.label_4)
        self.machine_type_box = QtGui.QComboBox(self.ayab_config)
        self.machine_type_box.setObjectName(_fromUtf8("machine_type_box"))
        self.machine_type_box.addItem(_fromUtf8(""))
        self.machine_type_box.addItem(_fromUtf8(""))
        self.verticalLayout.addWidget(self.machine_type_box)
        self.configure_button = QtGui.QPushButton(self.ayab_config)
        self.configure_button.setObjectName(_fromUtf8("configure_button"))
        self.verticalLayout.addWidget(self.configure_button)
        DockWidget.setWidget(self.ayab_config)

        self.retranslateUi(DockWidget)
        QtCore.QMetaObject.connectSlotsByName(DockWidget)

    def retranslateUi(self, DockWidget):
        DockWidget.setWindowTitle(_translate("DockWidget", "AYAB", None))
        self.groupBox.setTitle(_translate("DockWidget", "Port Selection", None))
        self.refresh_ports_button.setText(_translate("DockWidget", "Refresh Ports", None))
        self.label_6.setText(_translate("DockWidget", "Colors", None))
        self.label_5.setText(_translate("DockWidget", "Start Line", None))
        self.start_line_edit.setPrefix(_translate("DockWidget", "line ", None))
        self.infRepeat_checkbox.setText(_translate("DockWidget", "Infinite Repeat", None))
        self.gbox_startneedle.setTitle(_translate("DockWidget", "Start Needle", None))
        self.start_needle_color.setItemText(0, _translate("DockWidget", "orange", None))
        self.start_needle_color.setItemText(1, _translate("DockWidget", "green", None))
        self.gbox_stopneedle.setTitle(_translate("DockWidget", "Stop Needle", None))
        self.stop_needle_color.setItemText(0, _translate("DockWidget", "green", None))
        self.stop_needle_color.setItemText(1, _translate("DockWidget", "orange", None))
        self.label_3.setText(_translate("DockWidget", "Alignment", None))
        self.alignment_combo_box.setItemText(0, _translate("DockWidget", "center", None))
        self.alignment_combo_box.setItemText(1, _translate("DockWidget", "left", None))
        self.alignment_combo_box.setItemText(2, _translate("DockWidget", "right", None))
        self.label_4.setText(_translate("DockWidget", "Machine Type", None))
        self.machine_type_box.setItemText(0, _translate("DockWidget", "single", None))
        self.machine_type_box.setItemText(1, _translate("DockWidget", "double", None))
        self.configure_button.setText(_translate("DockWidget", "2. Configure", None))




# -*- coding: utf-8 -*-
# This file is part of AYAB.
#
#    AYAB is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    AYAB is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with AYAB.  If not, see <http://www.gnu.org/licenses/>.
#
#    Copyright 2013, 2014 Sebastian Oliva, Christian Obersteiner, Andreas Müller
#    https://bitbucket.org/chris007de/ayab-apparat/

from ayab_communication import AyabCommunication
import ayab_image
import time
import logging
import os
from ayab.plugins.knitting_plugin import KnittingPlugin
from PyQt4 import QtGui, QtCore

from ayab_options import Ui_DockWidget
import serial.tools.list_ports


class AyabPluginControl(KnittingPlugin):

  def onknit(self, e):
    logging.debug("called onknit on AyabPluginControl")
    self.__knitImage(self.__image, self.conf)
    self.finish()

  def onconfigure(self, e):
    logging.debug("called onconfigure on AYAB Knitting Plugin")
    #print ', '.join("%s: %s" % item for item in vars(e).items())
    #FIXME: substitute setting parent_ui from self.__parent_ui
    #self.__parent_ui = e.event.parent_ui
    parent_ui = self.__parent_ui

    #Start to knit with the bottom first
    pil_image = parent_ui.pil_image.rotate(180)

    conf = self.get_configuration_from_ui(parent_ui)
    #TODO: detect if previous conf had the same image to avoid re-generating.

    try:
      self.__image = ayab_image.ayabImage(pil_image, self.conf["num_colors"])
    except:
      self.__notify_user("You need to set an image.", "error")
      return

    if conf.get("start_needle") and conf.get("stop_needle"):
      self.__image.setKnitNeedles(conf.get("start_needle"), conf.get("stop_needle"))
      if conf.get("alignment"):
        self.__image.setImagePosition(conf.get("alignment"))
    if conf.get("start_line"):
      self.__image.setStartLine(conf.get("start_line"))

    if self.validate_configuration(conf):
      parent_ui.ui.knit_button.setEnabled(True)
      self.__emit_progress(0, 0, self.__image.imgHeight())
    return

  def validate_configuration(self, conf):
    if conf.get("start_needle") and conf.get("stop_needle"):
      if conf.get("start_needle") > conf.get("stop_needle"):
        self.__notify_user("Invalid needle start and end.", "warning")
        return False
    if conf.get("start_line") > self.__image.imgHeight():
      self.__notify_user("Start Line is larger than the image.")
      return False

    if conf.get("portname") == '':
      self.__notify_user("Please choose a valid port.")
      return False

    return True

  def onfinish(self, e):
    logging.info("Finished Knitting.")
    self.__close_serial()
    self.__parent_ui.resetUI()
    self.__parent_ui.emit(QtCore.SIGNAL('updateProgress(int,int,int)'), 0, 0, 0)

  def cancel(self):
    self._knitImage = False
    #self.finish()

  def __close_serial(self):
    try:
      self.__ayabCom.close_serial()
      logging.debug("Closing Serial port successful.")
    except:
      logging.debug("Closing Serial port failed. Was it ever open?")

  def onerror(self, e):
    #TODO add message info from event
    logging.error("Error while Knitting.")
    self.__close_serial()

  def __wait_for_user_action(self, message="", message_type="info"):
    """Sends the display_blocking_pop_up_signal QtSignal to main GUI thread, blocking it."""
    self.__parent_ui.emit(QtCore.SIGNAL('display_blocking_pop_up_signal(QString, QString)'), message, message_type)

  def __notify_user(self, message="", message_type="info"):
    """Sends the display_pop_up_signal QtSignal to main GUI thread, not blocking it."""
    self.__parent_ui.emit(QtCore.SIGNAL('display_pop_up_signal(QString, QString)'), message, message_type)

  def __emit_progress(self, percent, done, total):
    """Sends the updateProgress QtSignal."""
    self.__parent_ui.emit(QtCore.SIGNAL('updateProgress(int,int,int)'), int(percent), int(done), int(total))

  def setup_ui(self, parent_ui):
    """Sets up UI elements from ayab_options.Ui_DockWidget in parent_ui."""
    self.set_translator()
    self.__parent_ui = parent_ui
    self.options_ui = Ui_DockWidget()
    self.dock = parent_ui.ui.knitting_options_dock  # findChild(QtGui.QDockWidget, "knitting_options_dock")
    self.options_ui.setupUi(self.dock)
    self.setup_behaviour_ui()

  def set_translator(self):
    dirname = os.path.dirname(__file__)
    self.translator = QtCore.QTranslator()
    self.translator.load(QtCore.QLocale.system(), "ayab_options", ".", dirname, ".qm")
    app = QtCore.QCoreApplication.instance()
    app.installTranslator(self.translator)

  def unset_translator(self):
    app = QtCore.QCoreApplication.instance()
    app.removeTranslator(self.translator)

  def populate_ports(self, combo_box=None, port_list=None):
    if not combo_box:
      combo_box = self.__parent_ui.findChild(QtGui.QComboBox, "serial_port_dropdown")
    if not port_list:
      port_list = self.getSerialPorts()

    combo_box.clear()

    def populate(combo_box, port_list):
      for item in port_list:
        #TODO: should display the info of the device.
        combo_box.addItem(item[0])
    populate(combo_box, port_list)


  def setup_behaviour_ui(self):
    """Connects methods to UI elements."""
    conf_button = self.options_ui.configure_button  # Used instead of findChild(QtGui.QPushButton, "configure_button")
    conf_button.clicked.connect(self.conf_button_function)


    self.populate_ports()
    refresh_ports = self.options_ui.refresh_ports_button
    refresh_ports.click.connect(self.populate_ports)

  def conf_button_function(self):
    self.configure()

  def cleanup_ui(self, parent_ui):
    """Cleans up UI elements inside knitting option dock."""
    #dock = parent_ui.knitting_options_dock
    dock = self.dock
    cleaner = QtCore.QObjectCleanupHandler()
    cleaner.add(dock.widget())
    self.__qw = QtGui.QWidget()
    dock.setWidget(self.__qw)
    self.unset_translator()

  def get_configuration_from_ui(self, ui):
    """Creates a configuration dict from the ui elements.

    Returns:
      dict: A dict with configuration.

    """

    self.conf = {}
    color_line_text = ui.findChild(QtGui.QSpinBox, "color_edit").value()
    self.conf["num_colors"] = int(color_line_text)
    start_line_text = ui.findChild(QtGui.QSpinBox, "start_line_edit").value()
    self.conf["start_line"] = int(start_line_text)

    start_needle_color = ui.findChild(QtGui.QComboBox, "start_needle_color").currentText()
    start_needle_text = ui.findChild(QtGui.QSpinBox, "start_needle_edit").value()

    if(start_needle_color == "orange"):
      self.conf["start_needle"] = 100 - int(start_needle_text)
    elif(start_needle_color == "green"):
      self.conf["start_needle"] = 99 + int(start_needle_text)

    stop_needle_color = ui.findChild(QtGui.QComboBox, "stop_needle_color").currentText()
    stop_needle_text = ui.findChild(QtGui.QSpinBox, "stop_needle_edit").value()

    if(stop_needle_color == "orange"):
      self.conf["stop_needle"] = 100 - int(stop_needle_text)
    elif(stop_needle_color == "green"):
      self.conf["stop_needle"] = 99 + int(stop_needle_text)


    alignment_text = ui.findChild(QtGui.QComboBox, "alignment_combo_box").currentText()
    self.conf["alignment"] = alignment_text

    self.conf["inf_repeat"] = \
        int(ui.findChild(QtGui.QCheckBox, "infRepeat_checkbox").isChecked())

    machine_type_text = ui.findChild(QtGui.QComboBox, "machine_type_box").currentText()
    self.conf["machine_type"] = str(machine_type_text)

    serial_port_text = ui.findChild(QtGui.QComboBox, "serial_port_dropdown").currentText()
    self.conf["portname"] = str(serial_port_text)
    # getting file location from textbox
    filename_text = ui.findChild(QtGui.QLineEdit, "filename_lineedit").text()
    self.conf["filename"] = str(filename_text)
    logging.debug(self.conf)
    ## Add more config options.
    return self.conf

  def getSerialPorts(self):
      """
      Returns a list of all USB Serial Ports
      """
      return list(serial.tools.list_ports.grep("USB"))

  def __init__(self):
    super(AyabPluginControl, self).__init__({})
    # KnittingPlugin.__init__(self)

    #Copying from ayab_control
    self.__API_VERSION = 0x03
    self.__ayabCom = AyabCommunication()

    self.__formerRequest = 0
    self.__lineBlock = 0

  def __del__(self):
    self.__close_serial()

###Copied from ayab_control
#####################################

  def __setBit(self, int_type, offset):
      mask = 1 << offset
      return(int_type | mask)

  def __setPixel(self, bytearray, pixel):
      numByte = int(pixel / 8)
      bytearray[numByte] = self.__setBit(
          int(bytearray[numByte]), pixel - (8 * numByte))
      return bytearray

  def __checkSerial(self):
        time.sleep(1)  # TODO if problems in communication, tweak here

        line = self.__ayabCom.read_line()

        if line != '':
            msgId = ord(line[0])
            if msgId == 0xC1:    # cnfStart
                # print "> cnfStart: " + str(ord(line[1]))
                return ("cnfStart", ord(line[1]))

            elif msgId == 0xC3:  # cnfInfo
                # print "> cnfInfo: Version=" + str(ord(line[1]))
                logging.debug("Detected device with API v" + str(ord(line[1])))
                return ("cnfInfo", ord(line[1]))

            elif msgId == 0x82:  # reqLine
                # print "> reqLine: " + str(ord(line[1]))
                return ("reqLine", ord(line[1]))

            else:
                self.__printError("unknown message: " + line[:])  # drop crlf
                return ("unknown", 0)
        return("none", 0)

  def __cnfLine(self, lineNumber):
        imgHeight = self.__image.imgHeight()
        color = 0
        indexToSend = 0
        sendBlankLine = False
        lastLine = 0x00

        # TODO optimize performance
        # initialize bytearray to 0x00
        bytes = bytearray(25)
        for x in range(0, 25):
            bytes[x] = 0x00

        if lineNumber < 256:
            # TODO some better algorithm for block wrapping
            # if the last requested line number was 255, wrap to next block of
            # lines
            if self.__formerRequest == 255 and lineNumber == 0:
                self.__lineBlock += 1
            # store requested line number for next request
            self.__formerRequest = lineNumber
            reqestedLine = lineNumber

            # adjust lineNumber with current block
            lineNumber = lineNumber \
                + (self.__lineBlock * 256)

            # when knitting infinitely, keep the requested 
            # lineNumber in its limits
            if self.__infRepeat:
              lineNumber = lineNumber % imgHeight

            #########################
            # decide which line to send according to machine type and amount of colors
            # singlebed, 2 color
            if self.__machineType == 'single' \
                    and self.__numColors == 2:

                # color is always 0 in singlebed,
                # because both colors are knitted at once
                color = 0

                # calculate imgRow
                imgRow = lineNumber + self.__startLine

                # 0   1   2   3   4 .. (imgRow)
                # |   |   |   |   |
                # 0 1 2 3 4 5 6 7 8 .. (imageExpanded)
                indexToSend = imgRow * 2

                # Check if the last line of the image was requested
                if imgRow == imgHeight - 1:
                    lastLine = 0x01

            # doublebed, 2 color
            elif self.__machineType == 'double' \
                    and self.__numColors == 2:

                # calculate imgRow
                imgRow = int(lineNumber / 2) + self.__startLine

                # 0 0 1 1 2 2 3 3 4 4 .. (imgRow)
                # 0 1 2 3 4 5 6 7 8 9 .. (lineNumber)
                # | |  X  | |  X  | |
                # 0 1 3 2 4 5 7 6 8 9 .. (imageExpanded)
                lenImgExpanded = len(self.__image.imageExpanded())
                indexToSend = self.__startLine * 2

                # TODO more beautiful algo
                if lineNumber % 4 == 1 or lineNumber % 4 == 2:
                    color = 1
                else:
                    color = 0

                if (lineNumber - 2) % 4 == 0:
                    indexToSend += lineNumber + 1

                elif (lineNumber - 2) % 4 == 1:
                    indexToSend += lineNumber - 1
                    if (imgRow == imgHeight - 1) \
                            and (indexToSend == lenImgExpanded - 2):
                        lastLine = 0x01
                else:
                    indexToSend += lineNumber
                    if (imgRow == imgHeight - 1) \
                            and (indexToSend == lenImgExpanded - 1):
                        lastLine = 0x01

            # doublebed, multicolor
            elif self.__machineType == 'double' \
                    and self.__numColors > 2:

                # calculate imgRow
                imgRow = int(
                    lineNumber / (self.__numColors * 2)) + self.__startLine

                if (lineNumber % 2) == 0:
                    color = (lineNumber / 2) % self.__numColors
                    indexToSend = (imgRow * self.__numColors) + color
                    logging.debug("COLOR" + str(color))
                else:
                    sendBlankLine = True

                # TODO Check assignment
                if imgRow == imgHeight - 1 \
                        and (indexToSend == lenImgExpanded - 1):
                    lastLine = 0x01
            #########################

            # assign pixeldata
            imgStartNeedle = self.__image.imgStartNeedle()
            if imgStartNeedle < 0:
              imgStartNeedle = 0

            imgStopNeedle = self.__image.imgStopNeedle()
            if imgStopNeedle > 199:
              imgStopNeedle = 199

            # set the bitarray
            if color == 0 \
                    and self.__machineType == 'double':
                for col in range(0, 200):
                    if col < imgStartNeedle \
                            or col > imgStopNeedle:
                        bytes = self.__setPixel(bytes, col)

            for col in range(0, self.__image.imgWidth()):
                pxl = (self.__image.imageExpanded())[indexToSend][col]
                # take the image offset into account
                if pxl == True and sendBlankLine == False:
                    bytes = self.__setPixel(
                        bytes, col + self.__image.imgStartNeedle())

            # TODO implement CRC8
            crc8 = 0x00

            # send line to machine
            if self.__infRepeat:
              self.__ayabCom.cnf_line(reqestedLine, bytes, 0, crc8)
            else:
              self.__ayabCom.cnf_line(reqestedLine, bytes, lastLine, crc8)
            
            # screen output
            msg = str((self.__image.imageExpanded())[indexToSend])
            msg += ' Image Row: ' + str(imgRow)
            msg += ' (indexToSend: ' + str(indexToSend)
            msg += ', reqLine: ' + str(reqestedLine)
            msg += ', lineNumber: ' + str(lineNumber)
            msg += ', lineBlock:' + str(self.__lineBlock) + ')'
            logging.debug(msg)
            #sending line progress to gui
            progress_int = 100 * float(imgRow)/self.__image.imgHeight()
            self.__emit_progress(progress_int, imgRow, imgHeight)

        else:
            logging.error("requested lineNumber out of range")

        if lastLine:
          if self.__infRepeat:
              self.__lineBlock = 0
              return 0 # keep knitting
          else:
              return 1  # image finished
        else:
            return 0  # keep knitting

  def __knitImage(self, pImage, pOptions):
      self.__formerRequest = 0
      self.__image = pImage
      self.__startLine = pImage.startLine()

      self.__numColors = pOptions["num_colors"]
      self.__machineType = pOptions["machine_type"]
      self.__infRepeat = pOptions["inf_repeat"]

      API_VERSION = self.__API_VERSION
      curState = 's_init'
      oldState = 'none'

      if not self.__ayabCom.open_serial(pOptions["portname"]):
          logging.error("Could not open serial port")
          return

      self._knitImage = True
      while self._knitImage:
          # TODO catch keyboard interrupts to abort knitting
          # TODO: port to state machine or similar.
          rcvMsg, rcvParam = self.__checkSerial()
          if curState == 's_init':
              if oldState != curState:
                  self.__ayabCom.req_info()

              if rcvMsg == 'cnfInfo':
                  if rcvParam == API_VERSION:
                      curState = 's_start'
                      self.__wait_for_user_action("Please init machine. (Set the carriage to mode KC-I or KC-II and move the carriage over the left turn mark).")
                  else:
                      self.__notify_user("Wrong API.")
                      logging.error("wrong API version: " + str(rcvParam)
                                        + (" (expected: )") + str(API_VERSION))
                      return

          if curState == 's_start':
              if oldState != curState:
                  self.__ayabCom.req_start(self.__image.knitStartNeedle(),
                                           self.__image.knitStopNeedle())

              if rcvMsg == 'cnfStart':
                  if rcvParam == 1:
                      curState = 's_operate'
                      self.__wait_for_user_action("Ready to Operate")
                  else:
                      self.__wait_for_user_action("Device not ready, configure and try again.")
                      logging.error("device not ready")
                      return

          if curState == 's_operate':
              if rcvMsg == 'reqLine':
                  imageFinished = self.__cnfLine(rcvParam)
                  if imageFinished:
                      curState = 's_finished'

          if curState == 's_finished':
              self.__wait_for_user_action("Image transmission finished. Please knit until you hear the double beep sound.")
              return

          oldState = curState

      return



# -*- coding: utf-8 -*-
# This file is part of AYAB.
#
#    AYAB is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    AYAB is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with AYAB.  If not, see <http://www.gnu.org/licenses/>.
#
#    Copyright 2014 Sebastian Oliva, Christian Obersteiner, Andreas Müller
#    https://bitbucket.org/chris007de/ayab-apparat/

'''Provides conveniency functions to aproximately resize an image using a rational continued fraction aproximation.'''

from PIL import Image
from math import sqrt, log, floor


def resize_image(image, width_proportion, height_proportion):
    width, height = image.size
    resized_image = image.transform(
        (int(width_proportion * width), int(height_proportion * height)),
        Image.AFFINE,
        (1./width_proportion, 0, 0, 0, 1./height_proportion, 0),
        Image.NEAREST)
    return resized_image


## Based on code from:
## https://groups.yahoo.com/neo/groups/tuning-math/conversations/topics/14958

def contfrac(a):
    '''Returns a list of terms for a continuated fraction.'''
    terms=[]
    count=0
    b=1
    while ((b != 0) and (count < 7)): ### 5 times, emprical accuracy measurement
    ### limit
        terms.append(floor(a/(b+0.0)))
        a,b = b, a % b
        count = count + 1
    return terms


def ra(x): ### 'rational approximation', or convergent
    '''Generator for rational aproximation tuples of the float provided.'''
    numerators=[0,1]
    denominators=[1,0]
    expansion=contfrac(x) ### call the contfrac function
    for num in expansion: ### [-1] and [-2] index 'previous'
        ### and 'previous-previous'
        numerators.append((num*numerators[-1])+numerators[-2])
        denominators.append((num*denominators[-1])+denominators[-2])

    for index in range(len(numerators)):
        yield (numerators[index], denominators[index])
        #print "%i/%i" % (numerators[index], denominators[index])

def get_rational_ratios(ratio):
    ratios_list = list(ra(ratio))
    return ratios_list[2::]



# -*- coding: utf-8 -*-
#This file is part of AYAB.
#
#    AYAB is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    AYAB is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with AYAB.  If not, see <http://www.gnu.org/licenses/>.
#
#    Copyright 2013 Christian Obersteiner, Andreas Müller
#    https://bitbucket.org/chris007de/ayab-apparat/

from PIL import Image

class ayabImage(object):
  def __init__(self, pil_image, pNumColors = 2):
    self.__numColors      = pNumColors

    self.__imgPosition    = 'center'
    self.__imgStartNeedle = '0'
    self.__imgStopNeedle  = '0'

    self.__knitStartNeedle = 0
    self.__knitStopNeedle  = 199

    self.__startLine  = 0

    self.__image = pil_image

    self.__image = self.__image.convert('L') # convert to 1 byte depth
    self.__updateImageData()

  def imageIntern(self):
    return self.__imageIntern

  def imageExpanded(self):
    return self.__imageExpanded

  def imgWidth(self):
    return self.__imgWidth

  def imgHeight(self):
    return self.__imgHeight

  def knitStartNeedle(self):
    return self.__knitStartNeedle

  def knitStopNeedle(self):
    return self.__knitStopNeedle

  def imgStartNeedle(self):
    return self.__imgStartNeedle

  def imgStopNeedle(self):
    return self.__imgStopNeedle

  def imgPosition(self):
    return self.__imgPosition

  def startLine(self):
    return self.__startLine

  def numColors(self):
    return self.__numColors


  def __updateImageData(self):
    self.__imgWidth   = self.__image.size[0]
    self.__imgHeight  = self.__image.size[1]

    self.__convertImgToIntern()
    self.__calcImgStartStopNeedles()


  def __convertImgToIntern(self):
    num_colors = self.__numColors
    clr_range  = float(256)/num_colors

    imgWidth   = self.__imgWidth
    imgHeight  = self.__imgHeight

    self.__imageIntern = \
      [[0 for i in range(imgWidth)] \
      for j in range(imgHeight)]
    self.__imageColors = \
      [[0 for i in range(num_colors)] \
      for j in range(imgHeight)]
    self.__imageExpanded = \
      [[0 for i in range(imgWidth)] \
      for j in range(num_colors*imgHeight)]

    # Distill image to x colors
    for row in range(0, imgHeight):
      for col in range(0, imgWidth):
        pxl = self.__image.getpixel((col, row))

        for color in range(0, num_colors):
          lowerBound = int(color*clr_range)
          upperBound = int((color+1)*clr_range)
          if pxl>=lowerBound and pxl<upperBound:
            # color map
            self.__imageIntern[row][col]    = color
            # amount of bits per color per line
            self.__imageColors[row][color]  += 1
            # colors separated per line
            self.__imageExpanded[(num_colors*row)+color][col] = 1

    #print(self.__imageIntern)
    #print(self.__imageColors)
    #print(self.__imageExpanded)


  def __calcImgStartStopNeedles(self):
    if self.__imgPosition == 'center':
        needleWidth = (self.__knitStopNeedle - self.__knitStartNeedle) +1
        self.__imgStartNeedle = (self.__knitStartNeedle + needleWidth/2) - self.__imgWidth/2
        self.__imgStopNeedle  = self.__imgStartNeedle + self.__imgWidth -1

    elif self.__imgPosition == 'left':
        self.__imgStartNeedle = self.__knitStartNeedle
        self.__imgStopNeedle  = self.__imgStartNeedle + self.__imgWidth

    elif self.__imgPosition == 'right':
        self.__imgStopNeedle  = self.__knitStopNeedle
        self.__imgStartNeedle = self.__imgStopNeedle - self.__imgWidth

    elif int(self.__imgPosition) > 0 and int(self.__imgPosition) < 200:
        self.__imgStartNeedle = int(self.__imgPosition)
        self.__imgStopNeedle  = self.__imgStartNeedle + self.__imgWidth

    else:
        return False
    return True

  def setNumColors(self, pNumColors):
      """
      sets the number of colors the be used for knitting
      """
      if pNumColors > 1 and pNumColors < 7:
        self.__numColors      = pNumColors
        self.__updateImageData()
      return

  def invertImage(self):
      """
      invert the pixels of the image
      """
      for y in range(0, self.__image.size[1]):
        for x in range(0, self.__image.size[0]):
          pxl = self.__image.getpixel((x, y))
          self.__image.putpixel((x,y),255-pxl)
      self.__updateImageData()
      return


  def rotateImage(self):
      """
      rotate the image 90 degrees clockwise
      """
      self.__image = self.__image.rotate(-90)

      self.__updateImageData()
      return


  def resizeImage(self, pNewWidth):
      """
      resize the image to a given width, keeping the aspect ratio
      """
      wpercent = (pNewWidth/float(self.__image.size[0]))
      hsize = int((float(self.__image.size[1])*float(wpercent)))
      self.__image = self.__image.resize((pNewWidth,hsize), Image.ANTIALIAS)

      self.__updateImageData()
      return


  def setKnitNeedles(self, pKnitStart, pKnitStop):
      """
      set the start and stop needle
      """
      if (pKnitStart < pKnitStop) \
          and pKnitStart >= 0 \
          and pKnitStop < 200:
        self.__knitStartNeedle = pKnitStart
        self.__knitStopNeedle  = pKnitStop

      self.__updateImageData()
      return


  def setImagePosition(self, pImgPosition):
      """
      set the position of the pattern
      """
      ok = False
      if pImgPosition == 'left' \
            or pImgPosition == 'center' \
            or pImgPosition == 'right':
        ok = True
      elif (int(pImgPosition) >= 0 and int(pImgPosition) < 200):
        ok = True

      if ok:
        self.__imgPosition = pImgPosition
        self.__updateImageData()
      return

  def setStartLine(self, pStartLine):
      """
      set the line where to start knitting
      """
      #Check if StartLine is in valid range (picture height)
      if pStartLine >= 0 \
            and pStartLine < self.__image.size[1]:
        self.__startLine = pStartLine
      return



# -*- coding: utf-8 -*-
# This file is part of AYAB.
#
#    AYAB is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    AYAB is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with AYAB.  If not, see <http://www.gnu.org/licenses/>.
#
#    Copyright 2013 Christian Obersteiner, Andreas Müller
#    https://bitbucket.org/chris007de/ayab-apparat/

"""Handles the serial communication protocol.

This module handles serial communication, currently works in a synchronous way.
AyabCommunication uses an internal PySerial.Serial object to connect to the device.
The initializer can also be overriden with a dummy serial object.
"""

import time
import serial

import logging


class AyabCommunication(object):
  """Class Handling the serial communication protocol."""

  def __init__(self, serial=None):
    """Creates an AyabCommunication object, with an optional serial-like object."""
    logging.basicConfig(level=logging.DEBUG)
    self.__logger = logging.getLogger(__name__)
    self.__ser = serial

  def __del__(self):
    """Handles on delete behaviour closing serial port object."""
    self.close_serial()

  def open_serial(self, pPortname=None):
    """Opens serial port communication with a portName."""
    if not self.__ser:
      self.__portname = pPortname
      try:
          self.__ser = serial.Serial(self.__portname, 115200)
          time.sleep(1)
      except:
        self.__logger.error("could not open serial port " + self.__portname)
        raise CommunicationException()
      return True

  def close_serial(self):
    """Closes serial port."""
    try:
      self.__ser.close()
      del(self.__ser)
      self.__ser = None
    except:
      #TODO: add message for closing serial failure.
      raise CommunicationException()

  def read_line(self):
    """Reads a line from serial communication."""
    line = bytes()
    if self.__ser:
      while self.__ser.inWaiting() > 0:
          line += self.__ser.read(1)
    return line

  def req_start(self, startNeedle, stopNeedle):
      """Sends a start message to the controller."""

      msg = chr(0x01)  # msg id
      msg += chr(int(startNeedle))
      msg += chr(int(stopNeedle))
      # print "< reqStart"
      self.__ser.write(msg + '\n\r')

  def req_info(self):
      """Sends a request for information to controller."""
      # print "< reqInfo"
      self.__ser.write(chr(0x03) + '\n\r')

  def cnf_line(self, lineNumber, lineData, flags, crc8):
      """Sends a line of data via the serial port.

      Sends a line of data to the serial port, all arguments are mandatory.
      The data sent here is parsed by the Arduino controller which sets the
      knitting needles accordingly.

      Args:
        lineNumber (int): The line number to be sent.
        lineData (bytes): The bytearray to be sent to needles.
        flags (bytes): The flags sent to the controller.
        crc8 (bytes, optional): The CRC-8 checksum for transmission.

      """

      msg = chr(0x42)                    # msg id
      msg += chr(lineNumber)              # line number
      msg += lineData                     # line data
      msg += chr(flags)                   # flags
      msg += chr(crc8)                    # crc8
      # print "< cnfLine"
      # print lineData
      self.__ser.write(msg + '\n\r')


class CommunicationException(Exception):
  pass



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'firmware_flash_ui.ui'
#
# Created by: PyQt4 UI code generator 4.10.3
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    def _fromUtf8(s):
        return s

try:
    _encoding = QtGui.QApplication.UnicodeUTF8
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig, _encoding)
except AttributeError:
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig)

class Ui_FirmwareFlashFrame(object):
    def setupUi(self, FirmwareFlashFrame):
        FirmwareFlashFrame.setObjectName(_fromUtf8("FirmwareFlashFrame"))
        FirmwareFlashFrame.resize(672, 493)
        FirmwareFlashFrame.setFrameShape(QtGui.QFrame.StyledPanel)
        FirmwareFlashFrame.setFrameShadow(QtGui.QFrame.Raised)
        self.horizontalLayout = QtGui.QHBoxLayout(FirmwareFlashFrame)
        self.horizontalLayout.setObjectName(_fromUtf8("horizontalLayout"))
        self.widget = QtGui.QWidget(FirmwareFlashFrame)
        self.widget.setObjectName(_fromUtf8("widget"))
        self.verticalLayout_2 = QtGui.QVBoxLayout(self.widget)
        self.verticalLayout_2.setMargin(0)
        self.verticalLayout_2.setObjectName(_fromUtf8("verticalLayout_2"))
        self.textBrowser = QtGui.QTextBrowser(self.widget)
        self.textBrowser.setHtml(_fromUtf8("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:\'Sans Serif\'; font-size:11pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:18px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:9pt; font-weight:600;\">Firmware Install Guide</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'Arial,sans-serif\'; font-size:14px; color:#333333; background-color:#ffffff;\">For Linux Users:</span><span style=\" font-family:\'Arial,sans-serif\'; font-size:14px; color:#333333;\"> <br />To be able to communicate with your Arduino, it might be necessary to add the rights for USB communication by adding your user to some groups:</span></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; background-color:#f5f5f5;\"><span style=\" font-family:\'Bitstream Vera Sans Mono,DejaVu Sans Mono,Monaco,Courier,monospace\'; font-size:12px; color:#333333; background-color:#f5f5f5;\">sudo usermod -a -G tty </span><span style=\" font-family:\'Bitstream Vera Sans Mono,DejaVu Sans Mono,Monaco,Courier,monospace\'; font-size:12px; font-weight:600; color:#333333;\">[</span><span style=\" font-family:\'Bitstream Vera Sans Mono,DejaVu Sans Mono,Monaco,Courier,monospace\'; font-size:12px; color:#333333;\">userName</span><span style=\" font-family:\'Bitstream Vera Sans Mono,DejaVu Sans Mono,Monaco,Courier,monospace\'; font-size:12px; font-weight:600; color:#333333;\">]</span></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; background-color:#f5f5f5;\"><span style=\" font-family:\'Bitstream Vera Sans Mono,DejaVu Sans Mono,Monaco,Courier,monospace\'; font-size:12px; color:#333333;\">sudo usermod -a -G dialout </span><span style=\" font-family:\'Bitstream Vera Sans Mono,DejaVu Sans Mono,Monaco,Courier,monospace\'; font-size:12px; font-weight:600; color:#333333;\">[</span><span style=\" font-family:\'Bitstream Vera Sans Mono,DejaVu Sans Mono,Monaco,Courier,monospace\'; font-size:12px; color:#333333;\">userName</span><span style=\" font-family:\'Bitstream Vera Sans Mono,DejaVu Sans Mono,Monaco,Courier,monospace\'; font-size:12px; font-weight:600; color:#333333;\">]</span></p>\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-family:\'Arial,sans-serif\'; font-size:14px; color:#333333;\"><br /></p></body></html>"))
        self.textBrowser.setObjectName(_fromUtf8("textBrowser"))
        self.verticalLayout_2.addWidget(self.textBrowser)
        self.horizontalLayout.addWidget(self.widget)
        self.verticalLayout = QtGui.QVBoxLayout()
        self.verticalLayout.setObjectName(_fromUtf8("verticalLayout"))
        self.label_3 = QtGui.QLabel(FirmwareFlashFrame)
        self.label_3.setObjectName(_fromUtf8("label_3"))
        self.verticalLayout.addWidget(self.label_3)
        self.port_combo_box = QtGui.QComboBox(FirmwareFlashFrame)
        self.port_combo_box.setObjectName(_fromUtf8("port_combo_box"))
        self.verticalLayout.addWidget(self.port_combo_box)
        self.label_4 = QtGui.QLabel(FirmwareFlashFrame)
        self.label_4.setObjectName(_fromUtf8("label_4"))
        self.verticalLayout.addWidget(self.label_4)
        self.hardware_list = QtGui.QListWidget(FirmwareFlashFrame)
        self.hardware_list.setObjectName(_fromUtf8("hardware_list"))
        self.verticalLayout.addWidget(self.hardware_list)
        self.label = QtGui.QLabel(FirmwareFlashFrame)
        self.label.setObjectName(_fromUtf8("label"))
        self.verticalLayout.addWidget(self.label)
        self.controller_list = QtGui.QListWidget(FirmwareFlashFrame)
        self.controller_list.setObjectName(_fromUtf8("controller_list"))
        self.verticalLayout.addWidget(self.controller_list)
        self.label_2 = QtGui.QLabel(FirmwareFlashFrame)
        self.label_2.setObjectName(_fromUtf8("label_2"))
        self.verticalLayout.addWidget(self.label_2)
        self.firmware_list = QtGui.QListWidget(FirmwareFlashFrame)
        self.firmware_list.setObjectName(_fromUtf8("firmware_list"))
        self.verticalLayout.addWidget(self.firmware_list)
        self.flash_firmware = QtGui.QPushButton(FirmwareFlashFrame)
        self.flash_firmware.setObjectName(_fromUtf8("flash_firmware"))
        self.flash_firmware.setEnabled(False)
        self.verticalLayout.addWidget(self.flash_firmware)
        spacerItem = QtGui.QSpacerItem(20, 40, QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem)
        self.horizontalLayout.addLayout(self.verticalLayout)

        self.retranslateUi(FirmwareFlashFrame)
        QtCore.QMetaObject.connectSlotsByName(FirmwareFlashFrame)

    def retranslateUi(self, FirmwareFlashFrame):
        FirmwareFlashFrame.setWindowTitle(_translate("FirmwareFlashFrame", "Firmware Flashing Utility", None))
        self.label_3.setText(_translate("FirmwareFlashFrame", "Port", None))
        self.label_4.setText(_translate("FirmwareFlashFrame", "Hardware List", None))
        self.label.setText(_translate("FirmwareFlashFrame", "Controller", None))
        self.label_2.setText(_translate("FirmwareFlashFrame", "Firmware Version", None))
        self.flash_firmware.setText(_translate("FirmwareFlashFrame", "Flash", None))


if __name__ == "__main__":
    import sys
    app = QtGui.QApplication(sys.argv)
    FirmwareFlashFrame = QtGui.QFrame()
    ui = Ui_FirmwareFlashFrame()
    ui.setupUi(FirmwareFlashFrame)
    FirmwareFlashFrame.show()
    sys.exit(app.exec_())




# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'ayab/ayab_gui.ui'
#
# Created: Mon Feb  9 22:25:25 2015
#      by: PyQt4 UI code generator 4.10.4
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    def _fromUtf8(s):
        return s

try:
    _encoding = QtGui.QApplication.UnicodeUTF8
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig, _encoding)
except AttributeError:
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig)

class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName(_fromUtf8("MainWindow"))
        MainWindow.resize(1008, 856)
        self.centralwidget = QtGui.QWidget(MainWindow)
        self.centralwidget.setObjectName(_fromUtf8("centralwidget"))
        self.gridLayout = QtGui.QGridLayout(self.centralwidget)
        self.gridLayout.setObjectName(_fromUtf8("gridLayout"))
        self.assistant_dock = QtGui.QScrollArea(self.centralwidget)
        self.assistant_dock.setEnabled(True)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Preferred, QtGui.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.assistant_dock.sizePolicy().hasHeightForWidth())
        self.assistant_dock.setSizePolicy(sizePolicy)
        self.assistant_dock.setMinimumSize(QtCore.QSize(280, 0))
        self.assistant_dock.setMaximumSize(QtCore.QSize(540, 16777215))
        self.assistant_dock.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.assistant_dock.setFrameShadow(QtGui.QFrame.Plain)
        self.assistant_dock.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.assistant_dock.setWidgetResizable(False)
        self.assistant_dock.setObjectName(_fromUtf8("assistant_dock"))
        self.dockWidgetContents = QtGui.QWidget()
        self.dockWidgetContents.setGeometry(QtCore.QRect(0, 0, 500, 700))
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Preferred, QtGui.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.dockWidgetContents.sizePolicy().hasHeightForWidth())
        self.dockWidgetContents.setSizePolicy(sizePolicy)
        self.dockWidgetContents.setObjectName(_fromUtf8("dockWidgetContents"))
        self.verticalLayout_5 = QtGui.QVBoxLayout(self.dockWidgetContents)
        self.verticalLayout_5.setSizeConstraint(QtGui.QLayout.SetFixedSize)
        self.verticalLayout_5.setObjectName(_fromUtf8("verticalLayout_5"))
        self.widget_imgload = QtGui.QWidget(self.dockWidgetContents)
        self.widget_imgload.setEnabled(True)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Preferred, QtGui.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.widget_imgload.sizePolicy().hasHeightForWidth())
        self.widget_imgload.setSizePolicy(sizePolicy)
        self.widget_imgload.setObjectName(_fromUtf8("widget_imgload"))
        self.verticalLayout = QtGui.QVBoxLayout(self.widget_imgload)
        self.verticalLayout.setSizeConstraint(QtGui.QLayout.SetMaximumSize)
        self.verticalLayout.setMargin(0)
        self.verticalLayout.setObjectName(_fromUtf8("verticalLayout"))
        self.label_3 = QtGui.QLabel(self.widget_imgload)
        self.label_3.setObjectName(_fromUtf8("label_3"))
        self.verticalLayout.addWidget(self.label_3)
        self.module_dropdown = QtGui.QComboBox(self.widget_imgload)
        self.module_dropdown.setObjectName(_fromUtf8("module_dropdown"))
        self.verticalLayout.addWidget(self.module_dropdown)
        self.filename_lineedit = QtGui.QLineEdit(self.widget_imgload)
        self.filename_lineedit.setEnabled(False)
        self.filename_lineedit.setText(_fromUtf8(""))
        self.filename_lineedit.setObjectName(_fromUtf8("filename_lineedit"))
        self.verticalLayout.addWidget(self.filename_lineedit)
        self.load_file_button = QtGui.QPushButton(self.widget_imgload)
        self.load_file_button.setObjectName(_fromUtf8("load_file_button"))
        self.verticalLayout.addWidget(self.load_file_button)
        self.verticalLayout_5.addWidget(self.widget_imgload)
        self.widget_optionsdock = QtGui.QWidget(self.dockWidgetContents)
        self.widget_optionsdock.setEnabled(False)
        self.widget_optionsdock.setObjectName(_fromUtf8("widget_optionsdock"))
        self.verticalLayout_2 = QtGui.QVBoxLayout(self.widget_optionsdock)
        self.verticalLayout_2.setSizeConstraint(QtGui.QLayout.SetMaximumSize)
        self.verticalLayout_2.setMargin(0)
        self.verticalLayout_2.setObjectName(_fromUtf8("verticalLayout_2"))
        self.line = QtGui.QFrame(self.widget_optionsdock)
        self.line.setFrameShape(QtGui.QFrame.HLine)
        self.line.setFrameShadow(QtGui.QFrame.Sunken)
        self.line.setObjectName(_fromUtf8("line"))
        self.verticalLayout_2.addWidget(self.line)
        self.knitting_options_dock = QtGui.QDockWidget(self.widget_optionsdock)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Maximum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.knitting_options_dock.sizePolicy().hasHeightForWidth())
        self.knitting_options_dock.setSizePolicy(sizePolicy)
        self.knitting_options_dock.setObjectName(_fromUtf8("knitting_options_dock"))
        self.dockWidgetContents_2 = QtGui.QWidget()
        self.dockWidgetContents_2.setObjectName(_fromUtf8("dockWidgetContents_2"))
        self.knitting_options_dock.setWidget(self.dockWidgetContents_2)
        self.verticalLayout_2.addWidget(self.knitting_options_dock)
        self.verticalLayout_5.addWidget(self.widget_optionsdock)
        self.line_2 = QtGui.QFrame(self.dockWidgetContents)
        self.line_2.setFrameShape(QtGui.QFrame.HLine)
        self.line_2.setFrameShadow(QtGui.QFrame.Sunken)
        self.line_2.setObjectName(_fromUtf8("line_2"))
        self.verticalLayout_5.addWidget(self.line_2)
        self.widget_knitcontrol = QtGui.QWidget(self.dockWidgetContents)
        self.widget_knitcontrol.setEnabled(True)
        self.widget_knitcontrol.setObjectName(_fromUtf8("widget_knitcontrol"))
        self.verticalLayout_3 = QtGui.QVBoxLayout(self.widget_knitcontrol)
        self.verticalLayout_3.setSizeConstraint(QtGui.QLayout.SetMaximumSize)
        self.verticalLayout_3.setMargin(0)
        self.verticalLayout_3.setObjectName(_fromUtf8("verticalLayout_3"))
        self.knit_button = QtGui.QPushButton(self.widget_knitcontrol)
        self.knit_button.setEnabled(False)
        self.knit_button.setObjectName(_fromUtf8("knit_button"))
        self.verticalLayout_3.addWidget(self.knit_button)
        self.cancel_button = QtGui.QPushButton(self.widget_knitcontrol)
        self.cancel_button.setEnabled(False)
        self.cancel_button.setObjectName(_fromUtf8("cancel_button"))
        self.verticalLayout_3.addWidget(self.cancel_button)
        self.verticalLayout_5.addWidget(self.widget_knitcontrol)
        self.assistant_dock.setWidget(self.dockWidgetContents)
        self.gridLayout.addWidget(self.assistant_dock, 0, 1, 3, 1)
        self.image_side_panel = QtGui.QVBoxLayout()
        self.image_side_panel.setObjectName(_fromUtf8("image_side_panel"))
        self.widget_6 = QtGui.QWidget(self.centralwidget)
        self.widget_6.setObjectName(_fromUtf8("widget_6"))
        self.horizontalLayout_2 = QtGui.QHBoxLayout(self.widget_6)
        self.horizontalLayout_2.setMargin(0)
        self.horizontalLayout_2.setObjectName(_fromUtf8("horizontalLayout_2"))
        self.label_4 = QtGui.QLabel(self.widget_6)
        self.label_4.setObjectName(_fromUtf8("label_4"))
        self.horizontalLayout_2.addWidget(self.label_4)
        self.dimensions_label = QtGui.QLabel(self.widget_6)
        self.dimensions_label.setText(_fromUtf8(""))
        self.dimensions_label.setObjectName(_fromUtf8("dimensions_label"))
        self.horizontalLayout_2.addWidget(self.dimensions_label)
        self.image_side_panel.addWidget(self.widget_6)
        self.image_pattern_view = QtGui.QGraphicsView(self.centralwidget)
        self.image_pattern_view.setObjectName(_fromUtf8("image_pattern_view"))
        self.image_side_panel.addWidget(self.image_pattern_view)
        self.progress_label = QtGui.QLabel(self.centralwidget)
        self.progress_label.setText(_fromUtf8(""))
        self.progress_label.setAlignment(QtCore.Qt.AlignCenter)
        self.progress_label.setObjectName(_fromUtf8("progress_label"))
        self.image_side_panel.addWidget(self.progress_label)
        self.progressBar = QtGui.QProgressBar(self.centralwidget)
        self.progressBar.setProperty("value", 0)
        self.progressBar.setObjectName(_fromUtf8("progressBar"))
        self.image_side_panel.addWidget(self.progressBar)
        self.gridLayout.addLayout(self.image_side_panel, 0, 0, 1, 1)
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtGui.QMenuBar(MainWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 1008, 25))
        self.menubar.setObjectName(_fromUtf8("menubar"))
        self.menuFile = QtGui.QMenu(self.menubar)
        self.menuFile.setObjectName(_fromUtf8("menuFile"))
        self.menuHelp = QtGui.QMenu(self.menubar)
        self.menuHelp.setObjectName(_fromUtf8("menuHelp"))
        self.menuTools = QtGui.QMenu(self.menubar)
        self.menuTools.setObjectName(_fromUtf8("menuTools"))
        self.menuImage_Actions = QtGui.QMenu(self.menubar)
        self.menuImage_Actions.setEnabled(False)
        self.menuImage_Actions.setObjectName(_fromUtf8("menuImage_Actions"))
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QtGui.QStatusBar(MainWindow)
        self.statusbar.setObjectName(_fromUtf8("statusbar"))
        MainWindow.setStatusBar(self.statusbar)
        self.actionLoad_AYAB_Firmware = QtGui.QAction(MainWindow)
        self.actionLoad_AYAB_Firmware.setObjectName(_fromUtf8("actionLoad_AYAB_Firmware"))
        self.actionQuit = QtGui.QAction(MainWindow)
        self.actionQuit.setObjectName(_fromUtf8("actionQuit"))
        self.actionOpen_Knitting_Project = QtGui.QAction(MainWindow)
        self.actionOpen_Knitting_Project.setObjectName(_fromUtf8("actionOpen_Knitting_Project"))
        self.actionAbout = QtGui.QAction(MainWindow)
        self.actionAbout.setObjectName(_fromUtf8("actionAbout"))
        self.actionHelp = QtGui.QAction(MainWindow)
        self.actionHelp.setObjectName(_fromUtf8("actionHelp"))
        self.actionInvert = QtGui.QAction(MainWindow)
        self.actionInvert.setObjectName(_fromUtf8("actionInvert"))
        self.actionMirror = QtGui.QAction(MainWindow)
        self.actionMirror.setObjectName(_fromUtf8("actionMirror"))
        self.actionRotate_Left = QtGui.QAction(MainWindow)
        self.actionRotate_Left.setObjectName(_fromUtf8("actionRotate_Left"))
        self.actionRotate_Right = QtGui.QAction(MainWindow)
        self.actionRotate_Right.setObjectName(_fromUtf8("actionRotate_Right"))
        self.actionSmart_Resize = QtGui.QAction(MainWindow)
        self.actionSmart_Resize.setObjectName(_fromUtf8("actionSmart_Resize"))
        self.actionVertical_Flip = QtGui.QAction(MainWindow)
        self.actionVertical_Flip.setObjectName(_fromUtf8("actionVertical_Flip"))
        self.menuFile.addAction(self.actionQuit)
        self.menuHelp.addAction(self.actionAbout)
        self.menuTools.addAction(self.actionLoad_AYAB_Firmware)
        self.menuImage_Actions.addAction(self.actionInvert)
        self.menuImage_Actions.addSeparator()
        self.menuImage_Actions.addAction(self.actionVertical_Flip)
        self.menuImage_Actions.addAction(self.actionMirror)
        self.menuImage_Actions.addSeparator()
        self.menuImage_Actions.addAction(self.actionRotate_Left)
        self.menuImage_Actions.addAction(self.actionRotate_Right)
        self.menuImage_Actions.addSeparator()
        self.menuImage_Actions.addAction(self.actionSmart_Resize)
        self.menubar.addAction(self.menuFile.menuAction())
        self.menubar.addAction(self.menuImage_Actions.menuAction())
        self.menubar.addAction(self.menuTools.menuAction())
        self.menubar.addAction(self.menuHelp.menuAction())

        self.retranslateUi(MainWindow)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        MainWindow.setWindowTitle(_translate("MainWindow", "All Yarns Are Beautiful", None))
        self.assistant_dock.setWindowTitle(_translate("MainWindow", "Asistant", None))
        self.label_3.setText(_translate("MainWindow", "Load Images and Mode", None))
        self.load_file_button.setText(_translate("MainWindow", "1. Load Image File", None))
        self.knitting_options_dock.setWindowTitle(_translate("MainWindow", "Knitting Options", None))
        self.knit_button.setText(_translate("MainWindow", "3. Knit!", None))
        self.cancel_button.setText(_translate("MainWindow", "Cancel Knitting", None))
        self.label_4.setText(_translate("MainWindow", "Image Dimensions", None))
        self.menuFile.setTitle(_translate("MainWindow", "File", None))
        self.menuHelp.setTitle(_translate("MainWindow", "Help", None))
        self.menuTools.setTitle(_translate("MainWindow", "Tools", None))
        self.menuImage_Actions.setTitle(_translate("MainWindow", "Image Actions", None))
        self.actionLoad_AYAB_Firmware.setText(_translate("MainWindow", "Load AYAB Firmware", None))
        self.actionQuit.setText(_translate("MainWindow", "Quit", None))
        self.actionOpen_Knitting_Project.setText(_translate("MainWindow", "Open Knitting Project", None))
        self.actionAbout.setText(_translate("MainWindow", "Help - About", None))
        self.actionHelp.setText(_translate("MainWindow", "Help", None))
        self.actionInvert.setText(_translate("MainWindow", "Invert", None))
        self.actionMirror.setText(_translate("MainWindow", "Mirror", None))
        self.actionRotate_Left.setText(_translate("MainWindow", "Rotate Left", None))
        self.actionRotate_Right.setText(_translate("MainWindow", "Rotate Right", None))
        self.actionSmart_Resize.setText(_translate("MainWindow", "Smart Resize", None))
        self.actionVertical_Flip.setText(_translate("MainWindow", "Vertical Flip", None))




# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'smart_resize.ui'
#
# Created by: PyQt4 UI code generator 4.10.3
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    def _fromUtf8(s):
        return s

try:
    _encoding = QtGui.QApplication.UnicodeUTF8
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig, _encoding)
except AttributeError:
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig)

class Ui_Dialog(object):
    def setupUi(self, Dialog):
        Dialog.setObjectName(_fromUtf8("Dialog"))
        Dialog.resize(668, 331)
        self.gridLayout = QtGui.QGridLayout(Dialog)
        self.gridLayout.setObjectName(_fromUtf8("gridLayout"))
        self.label_3 = QtGui.QLabel(Dialog)
        self.label_3.setObjectName(_fromUtf8("label_3"))
        self.gridLayout.addWidget(self.label_3, 4, 4, 1, 1)
        self.label_2 = QtGui.QLabel(Dialog)
        self.label_2.setObjectName(_fromUtf8("label_2"))
        self.gridLayout.addWidget(self.label_2, 1, 4, 1, 1)
        self.label = QtGui.QLabel(Dialog)
        self.label.setObjectName(_fromUtf8("label"))
        self.gridLayout.addWidget(self.label, 0, 0, 1, 1)
        self.label_7 = QtGui.QLabel(Dialog)
        self.label_7.setObjectName(_fromUtf8("label_7"))
        self.gridLayout.addWidget(self.label_7, 3, 0, 1, 1)
        self.vertical_stitches_label = QtGui.QLabel(Dialog)
        self.vertical_stitches_label.setText(_fromUtf8(""))
        self.vertical_stitches_label.setObjectName(_fromUtf8("vertical_stitches_label"))
        self.gridLayout.addWidget(self.vertical_stitches_label, 2, 5, 1, 1)
        self.label_6 = QtGui.QLabel(Dialog)
        self.label_6.setObjectName(_fromUtf8("label_6"))
        self.gridLayout.addWidget(self.label_6, 0, 3, 1, 1)
        spacerItem = QtGui.QSpacerItem(20, 40, QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Expanding)
        self.gridLayout.addItem(spacerItem, 5, 4, 1, 1)
        spacerItem1 = QtGui.QSpacerItem(40, 20, QtGui.QSizePolicy.MinimumExpanding, QtGui.QSizePolicy.Minimum)
        self.gridLayout.addItem(spacerItem1, 5, 5, 1, 1)
        self.calculated_height_label = QtGui.QLabel(Dialog)
        self.calculated_height_label.setText(_fromUtf8(""))
        self.calculated_height_label.setObjectName(_fromUtf8("calculated_height_label"))
        self.gridLayout.addWidget(self.calculated_height_label, 4, 5, 1, 1)
        self.label_9 = QtGui.QLabel(Dialog)
        self.label_9.setObjectName(_fromUtf8("label_9"))
        self.gridLayout.addWidget(self.label_9, 2, 0, 1, 1)
        self.buttonBox = QtGui.QDialogButtonBox(Dialog)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtGui.QDialogButtonBox.Cancel|QtGui.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName(_fromUtf8("buttonBox"))
        self.gridLayout.addWidget(self.buttonBox, 6, 4, 1, 1)
        self.ratio_label = QtGui.QLabel(Dialog)
        self.ratio_label.setText(_fromUtf8(""))
        self.ratio_label.setObjectName(_fromUtf8("ratio_label"))
        self.gridLayout.addWidget(self.ratio_label, 0, 2, 1, 1)
        self.ratios_list = QtGui.QListWidget(Dialog)
        self.ratios_list.setObjectName(_fromUtf8("ratios_list"))
        self.gridLayout.addWidget(self.ratios_list, 1, 3, 5, 1)
        self.width_spinbox = QtGui.QDoubleSpinBox(Dialog)
        self.width_spinbox.setObjectName(_fromUtf8("width_spinbox"))
        self.gridLayout.addWidget(self.width_spinbox, 2, 2, 1, 1)
        self.label_8 = QtGui.QLabel(Dialog)
        self.label_8.setObjectName(_fromUtf8("label_8"))
        self.gridLayout.addWidget(self.label_8, 1, 0, 1, 3)
        self.calculated_width_label = QtGui.QLabel(Dialog)
        self.calculated_width_label.setText(_fromUtf8(""))
        self.calculated_width_label.setObjectName(_fromUtf8("calculated_width_label"))
        self.gridLayout.addWidget(self.calculated_width_label, 3, 5, 1, 1)
        self.label_5 = QtGui.QLabel(Dialog)
        self.label_5.setObjectName(_fromUtf8("label_5"))
        self.gridLayout.addWidget(self.label_5, 3, 4, 1, 1)
        self.label_4 = QtGui.QLabel(Dialog)
        self.label_4.setObjectName(_fromUtf8("label_4"))
        self.gridLayout.addWidget(self.label_4, 2, 4, 1, 1)
        self.height_spinbox = QtGui.QDoubleSpinBox(Dialog)
        self.height_spinbox.setObjectName(_fromUtf8("height_spinbox"))
        self.gridLayout.addWidget(self.height_spinbox, 3, 2, 1, 1)
        self.horizontal_stitches_label = QtGui.QLabel(Dialog)
        self.horizontal_stitches_label.setText(_fromUtf8(""))
        self.horizontal_stitches_label.setObjectName(_fromUtf8("horizontal_stitches_label"))
        self.gridLayout.addWidget(self.horizontal_stitches_label, 1, 5, 1, 1)

        self.retranslateUi(Dialog)
        QtCore.QObject.connect(self.buttonBox, QtCore.SIGNAL(_fromUtf8("accepted()")), Dialog.accept)
        QtCore.QObject.connect(self.buttonBox, QtCore.SIGNAL(_fromUtf8("rejected()")), Dialog.reject)
        QtCore.QMetaObject.connectSlotsByName(Dialog)

    def retranslateUi(self, Dialog):
        Dialog.setWindowTitle(_translate("Dialog", "Dialog", None))
        self.label_3.setText(_translate("Dialog", "Calculated Height", None))
        self.label_2.setText(_translate("Dialog", "Calculated Horizontal Stitches", None))
        self.label.setText(_translate("Dialog", "Ratio", None))
        self.label_7.setText(_translate("Dialog", "Physical Height", None))
        self.label_6.setText(_translate("Dialog", "Proposed Aproximate Ratios", None))
        self.label_9.setText(_translate("Dialog", "Physical Width", None))
        self.label_8.setText(_translate("Dialog", "Input Width and Height in cm", None))
        self.label_5.setText(_translate("Dialog", "Calculated Width", None))
        self.label_4.setText(_translate("Dialog", "Calculated Vertical Stitches", None))


if __name__ == "__main__":
    import sys
    app = QtGui.QApplication(sys.argv)
    Dialog = QtGui.QDialog()
    ui = Ui_Dialog()
    ui.setupUi(Dialog)
    Dialog.show()
    sys.exit(app.exec_())




# -*- coding: utf-8 -*-
# This file is part of AYAB.


from PyQt4 import QtGui
from PyQt4.QtGui import QFrame
from PyQt4 import QtCore

import serial
import serial.tools.list_ports
import json
import logging
import os
import platform
import subprocess

from firmware_flash_ui import Ui_FirmwareFlashFrame


class FirmwareFlash(QFrame):

    def __init__(self, parent_ui):
      #TODO: add creator that does not depend from super to ease testing.
      super(FirmwareFlash, self).__init__(None)

      self.__parent_ui = parent_ui

      self.ui = Ui_FirmwareFlashFrame()
      self.ui.setupUi(self)

      self.load_ports()
      self.load_json()

      self.ui.hardware_list.itemClicked[QtGui.QListWidgetItem].connect(self.hardware_item_activated)
      self.ui.controller_list.itemClicked[QtGui.QListWidgetItem].connect(self.controller_item_activated)
      self.ui.firmware_list.itemClicked[QtGui.QListWidgetItem].connect(self.firmware_item_activated)
      self.ui.flash_firmware.clicked.connect(self.execute_flash_command)

    def display_blocking_pop_up(self, message="", message_type="info"):
      logging.debug("message emited: '{}'".format(message))
      box_function = {
          "info": QtGui.QMessageBox.information,
          "warning": QtGui.QMessageBox.warning,
          "error": QtGui.QMessageBox.critical,
      }
      message_box_function = box_function.get(message_type, QtGui.QMessageBox.warning)
      ret = message_box_function(
          self,
          "AYAB",
          message,
          QtGui.QMessageBox.AcceptRole,
          QtGui.QMessageBox.AcceptRole)
      if ret == QtGui.QMessageBox.AcceptRole:
          return True

    def load_json(self):
      self.json_object = self.parse_json("")
      self.add_items_from_json_object(self.json_object)

    def parse_json(self, json_string):
      x = """{"kh910": {"uno": [{"url": "/", "version": "0.7 (latest)", "file": "kh910_uno_0-7.hex"}, \
                                {"url": "/", "version": "0.6", "file": "kh910_uno_0-6.hex"}], \
                        "mega2560": [{"url": "/", "version": "0.7 (latest)", "file": "kh910_mega_0-7.hex"}, \
                                      {"url": "/", "version": "0.6", "file": "kh910_mega_0-6.hex"}] \
                        }, \
              "kh930": {"uno": [{"url": "/", "version": "0.7 (latest)", "file": "kh930_uno_0-7.hex"}, \
                                {"url": "/", "version": "0.6", "file": "kh930_uno_0-6.hex"}], \
                        "mega2560": [{"url": "/", "version": "0.7 (latest)", "file": "kh930_mega_0-7.hex"}, \
                                      {"url": "/", "version": "0.6", "file": "kh930_mega_0-6.hex"}] \
                        }, \
              "hardware_test": {"uno": [{"url": "/", "version": "0.7 (latest)", "file": "hwtest_uno_0-7.hex"}, \
                                        {"url": "/", "version": "0.6", "file": "hwtest_uno_0-6.hex"}], \
                                "mega2560": [{"url": "/", "version": "0.7 (latest)", "file": "hwtest_mega_0-7.hex"}, \
                                              {"url": "/", "version": "0.6", "file": "hwtest_mega_0-6.hex"}] \
                                }}"""
      json_string = x
      return json.loads(json_string)

    def add_items_from_json_object(self, json_object):
      repo = json_object
      for hardware_device in repo:
        self.add_hardware_to_list(hardware_device)
        #for controller in repo.get(hardware_device, []):
          #self.add_controller_to_list(controller)
          #for firmware in repo[hardware_device][controller]:
            #self.add_firmware_dict_to_list(firmware)

    def hardware_item_activated(self, hardware_qitem):
      '''Signal on hardware_list activated. Triggers loading of controllers.'''
      logging.debug("selected "+hardware_qitem.text())
      self.load_controllers(hardware_qitem.text())

    def controller_item_activated(self, control_qitem):
      '''Signal on controller_list activated. Triggers loading of firmwares.'''
      logging.debug("selected "+control_qitem.text())
      hardware_loaded_qitem = self.ui.hardware_list.currentItem()
      self.load_firmware(hardware_loaded_qitem.text(), control_qitem.text())

    def firmware_item_activated(self, firmware_qitem):
      '''Signal on firmware_list activated.'''
      logging.debug("selected firmware qitem" +firmware_qitem.text())
      self.ui.flash_firmware.setEnabled(True)

    def load_controllers(self, hardware_qstring):
      self.clean_controller_list()
      repo = self.json_object
      hardware_string = unicode(hardware_qstring)
      for controller in repo.get(hardware_string, []):
        self.add_controller_to_list(controller)

    def load_firmware(self, hardware_qstring, controller_qstring):
      self.clean_firmware_list()
      hardware_key = unicode(hardware_qstring)
      controller_key = unicode(controller_qstring)
      repo = self.json_object
      for firmware in repo[hardware_key][controller_key]:
        self.add_firmware_dict_to_list(firmware)
      #print hardware_key
      #print controller_key

    def clean_hardware_list(self):
      self.__clean_QListWidget(self.ui.hardware_list)

    def clean_controller_list(self):
      self.__clean_QListWidget(self.ui.controller_list)

    def clean_firmware_list(self):
      self.__clean_QListWidget(self.ui.firmware_list)

    def __clean_QListWidget(self, qlistw):
      qlistw.clear()

    def add_hardware_to_list(self, hardware_device):
      logging.debug("Hardware Device "+ hardware_device)
      self.ui.hardware_list.addItem(hardware_device)

    def add_controller_to_list(self, controller):
      logging.debug("Controller "+ controller)
      self.ui.controller_list.addItem(controller)

    def add_firmware_dict_to_list(self, firmware):
      ## Could add more info to display, such as date.
      version = firmware.get("version", "unspecified version")
      logging.debug("firmware" + firmware.get("version"))
      self.ui.firmware_list.addItem(version)

    def execute_flash_command(self):
      os_name = platform.system()
      base_dir = os.path.dirname(__file__)
      port = self.ui.port_combo_box.currentText()
      hardware_name = unicode(self.ui.hardware_list.currentItem().text())
      controller_name = unicode(self.ui.controller_list.currentItem().text())
      firmware_key = unicode(self.ui.firmware_list.currentItem().text())
      firmware_name = "firmware.hex"
      for firmware in self.json_object[hardware_name][controller_name]:
        if firmware.get("version") == firmware_key:
          firmware_name = firmware.get("file")

      command = self.generate_command_with_options(base_dir,
                                                   os_name,
                                                   port,
                                                   hardware_name,
                                                   controller_name,
                                                   firmware_name,
                                                   )

      try:
        value = subprocess.call(command, shell=True)
        if value == 0:
          logging.info("Flashing Done!")
          self.display_blocking_pop_up("Flashing Done!")
        else:
          logging.info("Error on flashing firmware.")
          self.display_blocking_pop_up("Error on flashing firmware.", message_type="error")
      except e:
        logging.info("Error on flashing firmware.")
        self.display_blocking_pop_up("Error on flashing firmware.", message_type="error")

    def generate_command_with_options(self, base_dir, os_name, port, hardware_name, controller_name, firmware_name):
      exe_file_dict = {
                        "Windows": os.path.join("firmware", ".\\avrdude.exe"),
                        "Linux": os.path.join("firmware", "avrdude"), #TODO, detect 64bit OS
                      }
      ## If unknown OS we assume avrdude is installed and on the PATH.
      exe_file = exe_file_dict.get(os_name, "avrdude")
      exe_route = os.path.join(base_dir, exe_file)
      conf_file = os.path.join(base_dir, "firmware","avrdude.conf")

      binary_file = os.path.join(base_dir, "firmware", hardware_name, controller_name, firmware_name)
      serial_port = port
      # List of Arduino controllers and their avrdude names.
      device_dict = {
          "mega2560": "m2560",
          "uno": "atmega328p",
      }
      device = device_dict.get(controller_name)

      programmer_dict = {
          "uno": "arduino",
          "mega2560": "wiring",
      }
      programmer = programmer_dict.get(controller_name, "wiring")

      ## avrdude command.
      ## http://www.ladyada.net/learn/avr/avrdude.html
      ## http://sharats.me/the-ever-useful-and-neat-subprocess-module.html
      exec_command = """{0} -v -p {1} -C "{2}" -c {3} -P {4} -b115200 -D -Uflash:w:"{5}":i """.format(
                     exe_route, device, conf_file, programmer, serial_port, binary_file)
      logging.debug(exec_command)
      return exec_command

    def load_ports(self):
      ports_list = self.getSerialPorts()
      def populate(ui, port_list):
        for item in port_list:
          ui.port_combo_box.addItem(item[0])
      populate(self.ui, ports_list)


    def getSerialPorts(self):
      """
      Returns a list of all USB Serial Ports
      """
      return list(serial.tools.list_ports.grep("USB"))



# -*- coding: utf-8 -*-
# This file is part of AYAB.
#
#    AYAB is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    AYAB is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with AYAB.  If not, see <http://www.gnu.org/licenses/>.
#
#    Copyright 2014 Sebastian Oliva, Christian Obersteiner, Andreas Müller
#    https://bitbucket.org/chris007de/ayab-apparat/


from yapsy.IPlugin import IPlugin
from fysom import Fysom


class KnittingPlugin(IPlugin, Fysom):
  '''A generic plugin implementing a state machine for knitting.

  Subclasses inherit the basic State Machine defined in __init__.
  '''

  def onknit(self, e):
    """Callback when state machine executes knit().

    Starts the knitting process, this is the only function call that can block indefinitely, as it is called from an instance
    of QThread, allowing for processes that require timing and/or blocking behaviour.
    """
    raise NotImplementedError(self.__NOT_IMPLEMENTED_ERROR.format("onknit. It is used for the main 'knitting loop'."))

  def onfinish(self, e):
    """Callback when state machine executes finish().

    When finish() gets called, the plugin is expected to be able to restore it's state back when configure() gets called.
    Finish should trigger a Process Completed notification so the user can operate accordingly.
    """
    raise NotImplementedError(self.__NOT_IMPLEMENTED_ERROR.format("onfinish. It is a callback that is called when knitting is over."))

  def onconfigure(self, e):
    """Callback when state machine executes configure(parent_ui = parent, options={})

    This state gets called to configure the plugin for knitting. It can either
    be called when first configuring the plugin, when an error had happened and
    reset is necessary. The parent ui is expected to hold an object with
    properties.

    Args:
      parent_ui: An object having already been set up by setup_ui.
    """
    raise NotImplementedError(self.__NOT_IMPLEMENTED_ERROR.format("onconfigure. It is used to configure the knitting plugin before starting."))

  def setup_ui(self, parent_ui):
    '''Sets up UI, usually as a child of parent_ui.ui.knitting_options_dock.

    While the whole parent_ui object is available for the plugin to modify, plugins authors are **strongly** encouraged to
    only manipulate the knitting_options_dock, plugins have full access to the parent UI as a way to fully customize the GUI experience.

    Args:
        parent_ui: A PyQt.QMainWindow with the property parent_ui.ui.knitting_options_dock, an instance of QDockWidget to hold the plugin's UI.
    '''
    raise NotImplementedError(self.__NOT_IMPLEMENTED_ERROR.format("setup_ui. It loads the knitting_options_dock panel ui for the plugin."))

  def cleanup_ui(self, ui):
    '''Cleans up and reverts changes to ui done by setup_ui.'''
    raise NotImplementedError(self.__NOT_IMPLEMENTED_ERROR.format("cleanup_ui. It cleans up the knitting_options_dock panel ui for the plugin."))

  def get_configuration_from_ui(self, ui):
    """Loads options dict with a given parent QtGui object. Required for save-load functionality.

    Returns:
      dict: A dict with configuration.
    
    """
    raise NotImplementedError(self.__NOT_IMPLEMENTED_ERROR.format("get_configuration_from_ui. It loads options with a given parent ui object."))

  def __init__(self, callbacks_dict):
    self.__NOT_IMPLEMENTED_ERROR = "Classes that inherit from KnittingPlugin should implment {0}"

    callbacks_dict = {
        'onknit': self.onknit,
        #'onknitting': self.onknitting,
        'onconfigure': self.onconfigure,
        'onfinish': self.onfinish,
        }
    Fysom.__init__(self,
        {'initial': 'activated',
         'events': [
             ## TODO: add more states for handling error management.
             {'name': 'configure', 'src': 'activated', 'dst': 'configured'},
             {'name': 'configure', 'src': 'configured', 'dst': 'configured'},
             {'name': 'configure', 'src': 'finished', 'dst': 'configured'},
             {'name': 'configure', 'src': 'error', 'dst': 'configured'},
             {'name': 'knit', 'src': 'configured', 'dst': 'knitting'},
             {'name': 'finish', 'src': 'knitting', 'dst': 'finished'},
             {'name': 'fail', 'src': 'knittng', 'dst': 'error'}
         ],
         'callbacks':  callbacks_dict
         })



#!/usr/bin/python2
# -*- coding: utf-8 -*-
#This file is part of AYAB.
#
#    AYAB is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    AYAB is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with AYAB.  If not, see <http://www.gnu.org/licenses/>.
#
#    Copyright 2014 Sebastian Oliva, Christian Obersteiner, Andreas Müller,
#    https://bitbucket.org/chris007de/ayab-apparat/

from __future__ import print_function
from setuptools import setup, find_packages
from setuptools.command.test import test as TestCommand
import io
import codecs
import os
import glob
import sys

try:
    import py2exe
except:
    #TODO: notify of missing functionality.
    pass

import ayab

here = os.path.abspath(os.path.dirname(__file__))

## Useful Docs
## https://wiki.python.org/moin/Distutils/Tutorial
## https://pythonhosted.org/setuptools/setuptools.html


def read(*filenames, **kwargs):
    encoding = kwargs.get('encoding', 'utf-8')
    sep = kwargs.get('sep', '\n')
    buf = []
    for filename in filenames:
        with io.open(filename, encoding=encoding) as f:
            buf.append(f.read())
    return sep.join(buf)


def read_requirements(file_name):
    file_ob = file(file_name)
    raw_requirements_list = file_ob.readlines()
    requirements_list = []
    for line in raw_requirements_list:
      l = line.strip(' \t\n\r')
      if not l.startswith("#"):
        requirements_list.append(l)
    return requirements_list

## This builds the long description from Readme file, should be rst.
long_description = read('README.md')  # TODO: Add 'CHANGES.txt'


class PyTest(TestCommand):
    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = []
        self.test_suite = True

    def run_tests(self):
        import pytest
        errcode = pytest.main(self.test_args)
        sys.exit(errcode)


def find_data_files(source, target, patterns):
    """Locates the specified data-files and returns the matches
    in a data_files compatible format.

    source is the root of the source data tree.
        Use '' or '.' for current directory.
    target is the root of the target data tree.
        Use '' or '.' for the distribution directory.
    patterns is a sequence of glob-patterns for the
        files you want to copy.
    """
    if glob.has_magic(source) or glob.has_magic(target):
        raise ValueError("Magic not allowed in src, target")
    ret = {}
    for pattern in patterns:
        pattern = os.path.join(source, pattern)
        for filename in glob.glob(pattern):
            if os.path.isfile(filename):
                targetpath = os.path.join(target, os.path.relpath(filename, source))
                path = os.path.dirname(targetpath)
                ret.setdefault(path, []).append(filename)
    return sorted(ret.items())


setup(
    name='ayab',
    version=ayab.__version__,
    url='http://ayab-knitting.com/',
    license='GNU GPLv3+',
    author=u'Christian Obersteiner, Andreas Müller, Sebastian Oliva',
    scripts=['bin/ayab'],
    windows=['bin/ayab'],
    skip_archive=True,
    tests_require=['pytest'],
    options={"py2exe": {
        "includes": ["sip"],
        "skip_archive": True
        }},
    data_files=find_data_files('ayab', 'ayab', [
        'README.md',
        'patterns/*',
        '*.ts',
        '*.yapsy-plugin',
        'plugins/*',
        'plugins/ayab_plugin/firmware/*',
        'translations/*',
        ## FIXME: check alternatives for this nasty manual include
        ## http://objectmix.com/python/115674-py2exe-distutils-how-include-tree-files.html
        'plugins/ayab_plugin/firmware/hardware_test/mega2560/*',
        'plugins/ayab_plugin/firmware/hardware_test/uno/*',
        'plugins/ayab_plugin/firmware/kh910/mega2560/*',
        'plugins/ayab_plugin/firmware/kh910/uno/*',
        'plugins/ayab_plugin/firmware/kh930/mega2560/*',
        'plugins/ayab_plugin/firmware/kh930/uno/*',
        ]),
    install_requires=[read_requirements("requirements.txt")],
    cmdclass={'test': PyTest},
    author_email='info@ayab-knitting.com',
    description='GUI for Machine assisted Knitting. Reference implementation for AYAB.',
    long_description=long_description,
    ## TODO: load plugins automatically.
    packages=['ayab',
              'ayab.plugins',
              'ayab.plugins.ayab_plugin',
              'ayab.plugins.dummy_knitting_plugin', ],
    include_package_data=True,
    platforms='any',
    # test_suite='ayab.tests',
    classifiers=[
        'Programming Language :: Python',
        'Development Status :: 4 - Beta',
        'Natural Language :: English',
        ],
    extras_require={'testing': ['pytest']}
)



# -*- coding: utf-8 -*-
# This file is part of AYAB.
#
#    AYAB is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    AYAB is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with AYAB.  If not, see <http://www.gnu.org/licenses/>.
#
#    Copyright 2014 Sebastian Oliva, Christian Obersteiner, Andreas Müller
#    https://bitbucket.org/chris007de/ayab-apparat/

"""Provides an Interface for users to operate AYAB using a GUI."""

import sys
import os
import logging

from PyQt4 import QtGui, QtCore
from PyQt4.QtGui import QMainWindow
from PyQt4.QtCore import QThread

from yapsy import PluginManager
from PIL import ImageQt
from PIL import Image
from fysom import FysomError

from ayab_gui import Ui_MainWindow
from plugins.ayab_plugin.firmware_flash import FirmwareFlash
from ayab_about import Ui_AboutForm

## Temporal serial imports.
import serial
import serial.tools.list_ports


logging.basicConfig(level=logging.DEBUG)


class GuiMain(QMainWindow):
    """GuiMain is the main object that handles the instance of AYAB's GUI from ayab_gui.UiForm .

    GuiMain inherits from QMainWindow and instanciates a window with the form components form ayab_gui.UiForm.
    """

    def __init__(self):
        super(GuiMain, self).__init__(None)

        self.image_file_route = None
        self.enabled_plugin = None

        self.ui = Ui_MainWindow()
        self.ui.setupUi(self)
        self.plugins_init()
        self.setupBehaviour()

    def plugins_init(self, is_reloading=False):
        # Disable DEBUG logging of yapsy
        logging.getLogger('yapsy').setLevel(logging.WARNING)

        if is_reloading:
          logging.info("Deactivating All Plugins")
          for pluginInfo in self.pm.getAllPlugins():
            self.pm.deactivatePluginByName(pluginInfo.name)
        route = get_route()
        self.pm = PluginManager.PluginManager(directories_list=[os.path.join(route, "plugins")],)

        self.pm.collectPlugins()
        for pluginInfo in self.pm.getAllPlugins():
          ## This stops the plugins marked as Disabled from being activated.
          if (not pluginInfo.details.has_option("Core", "Disabled")):
            plugin_name = pluginInfo.name
            self.pm.activatePluginByName(plugin_name)
            self.add_plugin_name_on_module_dropdown(plugin_name)
            logging.info("Plugin {0} activated".format(plugin_name))
        ## Setting AYAB as the default value
        ## TODO: better way of setting ayab as default plugin.
        self.set_enabled_plugin("AYAB")

    def add_plugin_name_on_module_dropdown(self, module_name):
        self.ui.module_dropdown.addItem(module_name)

    def set_enabled_plugin(self, plugin_name=None):
        """Enables plugin, sets up gui and returns the plugin_object from the plugin selected on module_dropdown."""
        try:
          if self.enabled_plugin:
            self.enabled_plugin.plugin_object.cleanup_ui(self)
        except:
          pass

        if not plugin_name:
            plugin_name = self.ui.module_dropdown.currentText()
        plugin_o = self.pm.getPluginByName(plugin_name)
        self.enabled_plugin = plugin_o

        try:
            self.enabled_plugin.plugin_object.setup_ui(self)
            logging.info("Set enabled_plugin as {0} - {1}".format(plugin_o, plugin_name))
        except:
            logging.info("no plugin object loaded")
        return plugin_o

    def updateProgress(self, progress, done=0, total=0):
        '''Updates the Progress Bar.'''
        self.ui.progressBar.setValue(progress)
        self.ui.progress_label.setText("{0}/{1}".format(done, total))

    def update_file_selected_text_field(self, route):
        ''''Sets self.image_file_route and ui.filename_lineedit to route.'''
        self.ui.filename_lineedit.setText(route)
        self.image_file_route = route

    def start_knitting_process(self):
        # Disable everythin which should not be touched
        # during knitting
        self.ui.menuTools.setEnabled(False)
        self.ui.widget_imgload.setEnabled(False)
        self.ui.menuImage_Actions.setEnabled(False)
        self.ui.widget_optionsdock.setEnabled(False)
        self.ui.knit_button.setEnabled(False)
        self.ui.cancel_button.setEnabled(True)

        self.gt = GenericThread(self.enabled_plugin.plugin_object.knit, parent_window=self)
        self.gt.start()

    def cancel_knitting_process(self):
        self.enabled_plugin.plugin_object.cancel()

    def resetUI(self):
        # (Re-)enable UI elements
        self.ui.menuTools.setEnabled(True)
        self.ui.widget_imgload.setEnabled(True)
        self.ui.menuImage_Actions.setEnabled(True)
        self.ui.widget_optionsdock.setEnabled(True)
        self.ui.knit_button.setEnabled(False)
        self.ui.cancel_button.setEnabled(False)

    def setupBehaviour(self):
        # Connecting UI elements.
        self.ui.load_file_button.clicked.connect(self.file_select_dialog)
        self.ui.module_dropdown.activated[str].connect(self.set_enabled_plugin)
        self.ui.knit_button.clicked.connect(self.start_knitting_process)
        self.ui.cancel_button.clicked.connect(self.cancel_knitting_process)
        self.ui.actionLoad_AYAB_Firmware.triggered.connect(self.generate_firmware_ui)
        self.ui.image_pattern_view.setDragMode(QtGui.QGraphicsView.ScrollHandDrag)
        # Connecting Signals.
        self.connect(self, QtCore.SIGNAL("updateProgress(int,int,int)"), self.updateProgress)
        self.connect(self, QtCore.SIGNAL("display_pop_up_signal(QString, QString)"), self.display_blocking_pop_up)
        # This blocks the other thread until signal is done
        self.connect(self, QtCore.SIGNAL("display_blocking_pop_up_signal(QString, QString)"), self.display_blocking_pop_up, QtCore.Qt.BlockingQueuedConnection)
        self.connect(self, QtCore.SIGNAL("display_blocking_pop_up_signal(QString)"), self.display_blocking_pop_up, QtCore.Qt.BlockingQueuedConnection)
        self.ui.actionQuit.triggered.connect(QtCore.QCoreApplication.instance().quit)
        self.ui.actionAbout.triggered.connect(self.open_about_ui)
        self.ui.actionMirror.triggered.connect(self.mirror_image)
        self.ui.actionInvert.triggered.connect(self.invert_image)
        self.ui.actionRotate_Left.triggered.connect(self.rotate_left)
        self.ui.actionRotate_Right.triggered.connect(self.rotate_right)
        self.ui.actionVertical_Flip.triggered.connect(self.flip_image)
        self.ui.actionSmart_Resize.triggered.connect(self.smart_resize)

    def load_image_from_string(self, image_str):
        """Loads an image into self.ui.image_pattern_view using a temporary QGraphicsScene"""
        self.pil_image = Image.open(image_str)
        self.load_pil_image_on_scene(self.pil_image)
        self.ui.widget_optionsdock.setEnabled(True)
        self.ui.menuImage_Actions.setEnabled(True)

    def load_pil_image_on_scene(self, image_obj):
        '''Loads the PIL image on a QtScene and sets it as the current scene on the Image View.'''
        width, height = image_obj.size
        self.__qt_image = ImageQt.ImageQt(image_obj)
        self.__qpixmap = QtGui.QPixmap.fromImage(self.__qt_image)
        self.__qscene = QtGui.QGraphicsScene()
        self.__qscene.addPixmap(self.__qpixmap)

        #l = QtCore.QLineF(0,0,100,100)
        #self.__qscene.addLine(l)

        self.set_dimensions_on_gui(width, height)



        qv = self.ui.image_pattern_view
        qv.resetTransform()
        qv.scale(2.0, 2.0)
        qv.setScene(self.__qscene)

    def set_dimensions_on_gui(self, width, height):
        text = u"{} - {}".format(width, height)
        self.ui.dimensions_label.setText(text)

    def display_blocking_pop_up(self, message="", message_type="info"):
        logging.debug("message emited: '{}'".format(message))
        box_function = {
            "info": QtGui.QMessageBox.information,
            "warning": QtGui.QMessageBox.warning,
            "error": QtGui.QMessageBox.critical,
        }
        message_box_function = box_function.get(message_type, QtGui.QMessageBox.warning)
        ret = message_box_function(
            self,
            "AYAB",
            message,
            QtGui.QMessageBox.AcceptRole,
            QtGui.QMessageBox.AcceptRole)
        if ret == QtGui.QMessageBox.AcceptRole:
            return True

    def conf_button_function(self):
        self.enabled_plugin.plugin_object.configure(self)

    def file_select_dialog(self):
        file_selected_route = QtGui.QFileDialog.getOpenFileName(self)
        self.update_file_selected_text_field(file_selected_route)
        self.load_image_from_string(str(file_selected_route))

    def generate_firmware_ui(self):
      self.__flash_ui = FirmwareFlash(self)
      self.__flash_ui.show()

    def open_about_ui(self):
        self.__AboutForm = QtGui.QFrame()
        self.__about_ui = Ui_AboutForm()
        self.__about_ui.setupUi(self.__AboutForm)
        self.__AboutForm.show()

    def invert_image(self):
        '''Public invert current Image function.'''
        self.apply_image_transform("invert")

    def mirror_image(self):
        '''Public mirror current Image function.'''
        self.apply_image_transform("mirror")

    def flip_image(self):
        '''Public mirror current Image function.'''
        self.apply_image_transform("flip")

    def rotate_left(self):
        '''Public rotate left current Image function.'''
        self.apply_image_transform("rotate", -90.0)

    def rotate_right(self):
        '''Public rotate right current Image function.'''
        self.apply_image_transform("rotate", 90.0)

    def smart_resize(self):
      '''Executes the smart resize process including dialog .'''
      dialog_result = self.__launch_get_start_smart_resize_dialog_result(self)
      if dialog_result:
        self.apply_image_transform("smart_resize", dialog_result)

    def apply_image_transform(self, transform_type, *args):
        '''Executes an image transform specified by key and args.

        Calls a function from transform_dict, forwarding args and the image,
        and replaces the QtImage on scene.
        '''
        transform_dict = {
            'invert': self.__invert_image,
            'mirror': self.__mirror_image,
            'flip': self.__flip_image,
            'rotate': self.__rotate_image,
            'smart_resize': self.__smart_resize_image,
        }
        transform = transform_dict.get(transform_type)
        image = self.pil_image
        if not image:
            return
        # Executes the transform function
        try:
          image = transform(image, args)
        except:
          logging.error("Error on executing transform")

        # Disable Knit Controls
        self.ui.widget_knitcontrol.setEnabled(False)

        # Update the view
        self.pil_image = image
        self.load_pil_image_on_scene(self.pil_image)

    def __smart_resize_image(self, image, args):
      '''Implement the smart resize processing. Ratio sent as a tuple of horizontal and vertical values.'''
      import knit_aware_resize
      wratio, hratio = args[0]  # Unpacks the first argument.
      logging.debug("resizing image with args: {0}".format(args))
      resized_image = knit_aware_resize.resize_image(image, wratio, hratio)
      return resized_image

    def __rotate_image(self, image, args):
        if not args:
            logging.debug("image not altered on __rotate_image.")
            return image
        rotated_image = image.rotate(args[0], expand=True)
        return rotated_image

    def __invert_image(self, image, args):
        import PIL.ImageChops
        inverted_image = PIL.ImageChops.invert(image)
        return inverted_image

    def __mirror_image(self, image, args):
        import PIL.ImageOps
        mirrored_image = PIL.ImageOps.mirror(image)
        return mirrored_image

    def __flip_image(self, image, args):
        import PIL.ImageOps
        flipped_image = PIL.ImageOps.flip(image)
        return flipped_image

    def __launch_get_start_smart_resize_dialog_result(self, parent):
        '''Processes dialog and returns a ratio tuple or False.'''
        import smart_resize
        import knit_aware_resize
        ##TODO: create smart_resize dialog
        ## Show dialog
        self.physical_width, self.physical_height = 0.0, 0.0
        self.ratio_tuple = 1.0, 1.0
        ratio = 0.0
        dialog = QtGui.QDialog()
        dialog.ui = smart_resize.Ui_Dialog()
        dialog.ui.setupUi(dialog)

        def calculate_ratio_value(height, width):
          '''Calculates the ratio value with given height and width.'''
          try:
            return height / width
          except:
            return 0.0

        def set_ratio_list(ratio):
          '''Sets the dialog ratio list to a list of aproximations of rational ratios that match it.'''
          dialog.ui.ratios_list.clear()
          self.ratio_list = knit_aware_resize.get_rational_ratios(ratio)
          for ratio in self.ratio_list:
            ratio_string = "{0:.0f} - {1:.0f}".format(ratio[0], ratio[1])
            dialog.ui.ratios_list.addItem(ratio_string)

        def set_ratio_value(ratio):
          dialog.ui.ratio_label.setText(u"{0:.2f}".format(ratio))
          set_ratio_list(ratio)

        def recalculate_ratio():
          ratio = calculate_ratio_value(self.physical_height, self.physical_width)
          set_ratio_value(ratio)
          logging.debug("Set Ratio to {}".format(ratio))

        def set_height_ratio(height_string):
          self.physical_height = float(height_string)
          recalculate_ratio()

        def set_width_ratio(width_string):
          self.physical_width = float(width_string)
          recalculate_ratio()

        def get_ratios_list_item_value(selected_value):
          '''Gets the value of the tuple corresponding to the selected ratio list.'''
          try:
            self.ratio_tuple = self.ratio_list[selected_value]
            recalculate_real_size(self.ratio_tuple)
            logging.debug(self.ratio_tuple)
          except IndexError:
            pass

        def recalculate_real_size(ratio_tuple):
          '''Updates the calculations on the Resize dialog.'''
          try:
            h, w = self.pil_image.size
            h_ratio, w_ratio = ratio_tuple
            horizontal_size_text, vertical_size_text = u"{0:.2f}".format(h * h_ratio), u"{0:.2f}".format(w * w_ratio)
            dialog.ui.horizontal_stitches_label.setText(horizontal_size_text)
            dialog.ui.vertical_stitches_label.setText(vertical_size_text)
            real_width_text, real_height_text = unicode(self.physical_height * h_ratio), unicode(self.physical_width * w_ratio)
            dialog.ui.calculated_width_label.setText(real_width_text)
            dialog.ui.calculated_height_label.setText(real_height_text)
          except:
            pass

        dialog.ui.height_spinbox.valueChanged[unicode].connect(set_height_ratio)
        dialog.ui.width_spinbox.valueChanged[unicode].connect(set_width_ratio)
        dialog.ui.ratios_list.currentRowChanged[int].connect(get_ratios_list_item_value)

        dialog_ok = dialog.exec_()
        #dialog.show()

        logging.debug(dialog_ok)
        if dialog_ok:
          print ratio
          return self.ratio_tuple
          #set variables to parent
        else:
          return False

    def getSerialPorts(self):
      """
      Returns a list of all USB Serial Ports
      """
      return list(serial.tools.list_ports.grep("USB"))


class GenericThread(QThread):
    '''A generic thread wrapper for functions on threads.'''

    def __init__(self, function, *args, **kwargs):
        QtCore.QThread.__init__(self)
        self.function = function
        self.args = args
        self.kwargs = kwargs

    def __del__(self):
        #self.join()
        self.wait()

    def run(self):
        try:
            self.function(*self.args, **self.kwargs)
        except FysomError as fe:
            logging.error(fe)
            parent = self.kwargs["parent_window"]
            parent.emit(QtCore.SIGNAL('display_blocking_pop_up_signal(QString, QString)'), QtGui.QApplication.translate("Form",
                        "Error on plugin action, be sure to configure before starting Knitting.", None), "error")
        return


def get_route():
  #if getattr(sys, 'frozen', False):
  #  route = sys._MEIPASS
  #  logging.debug("Loading AYAB from pyinstaller.")
  #  return route
  #else:
    filename = os.path.dirname(__file__)
    logging.debug("Loading AYAB from normal package structure.")
    return filename


def run():
  translator = QtCore.QTranslator()
  ## Loading ayab_gui main translator.
  translator.load(QtCore.QLocale.system(), "ayab_gui", ".", os.path.join(get_route(), "translations"), ".qm")
  app = QtGui.QApplication(sys.argv)
  app.installTranslator(translator)
  window = GuiMain()
  window.show()
  sys.exit(app.exec_())

if __name__ == '__main__':
  run()



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'ayab_about.ui'
#
# Created: Sat Feb  7 20:47:30 2015
#      by: PyQt4 UI code generator 4.10.4
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    def _fromUtf8(s):
        return s

try:
    _encoding = QtGui.QApplication.UnicodeUTF8
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig, _encoding)
except AttributeError:
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig)

class Ui_AboutForm(object):
    def setupUi(self, AboutForm):
        AboutForm.setObjectName(_fromUtf8("AboutForm"))
        AboutForm.resize(583, 312)
        AboutForm.setAutoFillBackground(False)
        AboutForm.setStyleSheet(_fromUtf8("background-color: rgb(255, 255, 255);"))
        self.verticalLayout = QtGui.QVBoxLayout(AboutForm)
        self.verticalLayout.setObjectName(_fromUtf8("verticalLayout"))
        self.label = QtGui.QLabel(AboutForm)
        font = QtGui.QFont()
        font.setPointSize(144)
        self.label.setFont(font)
        self.label.setText(_fromUtf8(""))
        self.label.setPixmap(QtGui.QPixmap(_fromUtf8(":/ayab_logo.jpg")))
        self.label.setScaledContents(True)
        self.label.setObjectName(_fromUtf8("label"))
        self.verticalLayout.addWidget(self.label)
        self.label_2 = QtGui.QLabel(AboutForm)
        self.label_2.setObjectName(_fromUtf8("label_2"))
        self.verticalLayout.addWidget(self.label_2)
        self.label_4 = QtGui.QLabel(AboutForm)
        self.label_4.setObjectName(_fromUtf8("label_4"))
        self.verticalLayout.addWidget(self.label_4)
        self.label_3 = QtGui.QLabel(AboutForm)
        self.label_3.setObjectName(_fromUtf8("label_3"))
        self.verticalLayout.addWidget(self.label_3)
        spacerItem = QtGui.QSpacerItem(20, 40, QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem)

        self.retranslateUi(AboutForm)
        QtCore.QMetaObject.connectSlotsByName(AboutForm)

    def retranslateUi(self, AboutForm):
        AboutForm.setWindowTitle(_translate("AboutForm", "About AYAB", None))
        self.label_2.setText(_translate("AboutForm", "All Yarns Are Beautiful", None))
        self.label_4.setText(_translate("AboutForm", "<html><head/><body><p><a href=\"http://ayab-knitting.com\"><span style=\" text-decoration: underline; color:#0000ff;\">http://ayab-knitting.com</span></a></p></body></html>", None))
        self.label_3.setText(_translate("AboutForm", "Version 0.7", None))

import resources_rc
